@@ -9,6 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2007      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -21,6 +22,7 @@
 #include <inttypes.h>
 #include "opal/util/output.h"
 #include "opal/util/if.h"
+#include "opal/util/show_help.h"
 #include "ompi/mca/pml/pml.h"
 #include "ompi/mca/btl/btl.h"
 #include "ompi/mca/btl/base/btl_base_error.h"
@@ -32,10 +34,23 @@
 #include "ompi/datatype/datatype.h" 
 #include "ompi/mca/mpool/base/base.h" 
 #include "ompi/mca/mpool/mpool.h" 
-#include "ompi/mca/mpool/openib/mpool_openib.h" 
+#include "ompi/mca/mpool/rdma/mpool_rdma.h"
+#include "orte/util/sys_info.h"
 #include <errno.h> 
 #include <string.h> 
 #include <math.h>
+#include <inttypes.h>
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+#ifdef HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif
+#ifdef HAVE_SYS_RESOURCE_H
+#include <sys/resource.h>
+#endif
+
+
 mca_btl_openib_module_t mca_btl_openib_module = {
     {
         &mca_btl_openib_component.super,
@@ -60,12 +75,44 @@ mca_btl_openib_module_t mca_btl_openib_module = {
         mca_btl_openib_send,
         mca_btl_openib_put,
         mca_btl_openib_get,
-        mca_btl_base_dump
+        mca_btl_base_dump,
+        NULL, /* mpool */
+        mca_btl_openib_register_error_cb /* error call back registration */
     }
 };
 
 int mca_btl_openib_size_queues( struct mca_btl_openib_module_t* openib_btl, size_t nprocs);
-int mca_btl_openib_create_cq_srq(mca_btl_openib_module_t *openib_btl);
+
+
+static void show_init_error(const char *file, int line, 
+                            const char *func, const char *dev) 
+{
+    if (ENOMEM == errno) {
+        int ret;
+        struct rlimit limit;
+        char *str_limit = NULL;
+
+        ret = getrlimit(RLIMIT_MEMLOCK, &limit);
+        if (0 != ret) {
+            asprintf(&str_limit, "Unknown");
+        } else if (limit.rlim_cur == RLIM_INFINITY) {
+            asprintf(&str_limit, "unlimited");
+        } else {
+            asprintf(&str_limit, "%ld", (long) limit.rlim_cur);
+        }
+
+        opal_show_help("help-mpi-btl-openib.txt", "init-fail-no-mem",
+                       true, orte_system_info.nodename, 
+                       file, line, func, dev, str_limit);
+
+        if (NULL != str_limit) free(str_limit);
+    } else {
+        opal_show_help("help-mpi-btl-openib.txt", "init-fail-create-q",
+                       true, orte_system_info.nodename, 
+                       file, line, func, strerror(errno), errno, dev);
+    }
+}
+
 
 /* 
  *  add a proc to this btl module 
@@ -80,53 +127,95 @@ int mca_btl_openib_add_procs(
     ompi_bitmap_t* reachable)
 {
     mca_btl_openib_module_t* openib_btl = (mca_btl_openib_module_t*)btl;
-    int i, rc;
+    int i,j, rc;
+    int rem_subnet_id_port_cnt;
+    int lcl_subnet_id_port_cnt = 0;
+    int btl_rank;
 
+    for(j=0; j < mca_btl_openib_component.ib_num_btls; j++){ 
+        if(mca_btl_openib_component.openib_btls[j].port_info.subnet_id
+           == openib_btl->port_info.subnet_id) { 
+            lcl_subnet_id_port_cnt++;
+            }
+        if(openib_btl == &(mca_btl_openib_component.openib_btls[j])) { 
+            btl_rank = j;
+        }
+    }
     for(i = 0; i < (int) nprocs; i++) {
         
         struct ompi_proc_t* ompi_proc = ompi_procs[i];
         mca_btl_openib_proc_t* ib_proc;
-        mca_btl_base_endpoint_t* ib_peer;
-
+        mca_btl_base_endpoint_t* endpoint;
+        
+        
         if(NULL == (ib_proc = mca_btl_openib_proc_create(ompi_proc))) {
             return OMPI_ERR_OUT_OF_RESOURCE;
         }
 
-        /*
-         * Check to make sure that the peer has at least as many interface 
-         * addresses exported as we are trying to use. If not, then 
-         * don't bind this PTL instance to the proc.
-         */
-
+        rem_subnet_id_port_cnt  = 0;
+        /* check if the remote proc has a reachable subnet first */
+        BTL_VERBOSE(("got %d port_infos \n", ib_proc->proc_port_count));
+        for(j = 0; j < (int) ib_proc->proc_port_count; j++){
+            BTL_VERBOSE(("got a subnet %016x\n",
+                         ib_proc->proc_ports[j].subnet_id));
+            if(ib_proc->proc_ports[j].subnet_id ==
+               openib_btl->port_info.subnet_id) {
+                BTL_VERBOSE(("Got a matching subnet!\n"));
+                rem_subnet_id_port_cnt ++;
+            }
+        }
+        if(!rem_subnet_id_port_cnt ) {
+            /* no use trying to communicate with this endpointlater */
+            BTL_VERBOSE(("No matching subnet id was found, moving on.. \n"));
+            continue;
+        }
+        
+        
+#if 0
+        num_endpoints = rem_subnet_id_port_cnt  / lcl_subnet_id_port_cnt + 
+            (btl_rank < (rem_subnet_id_port_cnt  / lcl_subnet_id_port_cnt)) ? 1:0;
+        
+#endif 
+        if(rem_subnet_id_port_cnt  < lcl_subnet_id_port_cnt && 
+           btl_rank >= rem_subnet_id_port_cnt ) { 
+            BTL_VERBOSE(("Not enough remote ports on this subnet id, moving on.. \n"));
+            continue;
+            
+        }
         OPAL_THREAD_LOCK(&ib_proc->proc_lock);
 
         /* The btl_proc datastructure is shared by all IB PTL
          * instances that are trying to reach this destination. 
          * Cache the peer instance on the btl_proc.
          */
-        ib_peer = OBJ_NEW(mca_btl_openib_endpoint_t);
-        if(NULL == ib_peer) {
+        endpoint = OBJ_NEW(mca_btl_openib_endpoint_t);
+        if(NULL == endpoint) {
             OPAL_THREAD_UNLOCK(&ib_proc->proc_lock);
             return OMPI_ERR_OUT_OF_RESOURCE;
         }
-
-        ib_peer->endpoint_btl = openib_btl;
-        ib_peer->subnet = openib_btl->port_info.subnet; 
-        rc = mca_btl_openib_proc_insert(ib_proc, ib_peer);
+            
+        endpoint->endpoint_btl = openib_btl;
+        endpoint->use_eager_rdma = openib_btl->hca->use_eager_rdma &
+            mca_btl_openib_component.use_eager_rdma;
+        endpoint->subnet_id = openib_btl->port_info.subnet_id; 
+        rc = mca_btl_openib_proc_insert(ib_proc, endpoint);
         if(rc != OMPI_SUCCESS) {
-            OBJ_RELEASE(ib_peer);
+            OBJ_RELEASE(endpoint);
             OPAL_THREAD_UNLOCK(&ib_proc->proc_lock);
             continue;
         }
-
+        
+        orte_pointer_array_add((orte_std_cntr_t*)&endpoint->index,
+                               openib_btl->endpoints, (void*)endpoint);
         ompi_bitmap_set_bit(reachable, i);
         OPAL_THREAD_UNLOCK(&ib_proc->proc_lock);
-        peers[i] = ib_peer;
+        
+        peers[i] = endpoint;
     }
     
     return mca_btl_openib_size_queues(openib_btl, nprocs);
-}
 
+}
 
 int mca_btl_openib_size_queues( struct mca_btl_openib_module_t* openib_btl, size_t nprocs) 
 {
@@ -134,7 +223,6 @@ int mca_btl_openib_size_queues( struct mca_btl_openib_module_t* openib_btl, size
     int first_time = (0 == openib_btl->num_peers);
     int rc;
     openib_btl->num_peers += nprocs; 
-#ifdef OMPI_MCA_BTL_OPENIB_HAVE_SRQ
     if(mca_btl_openib_component.use_srq) { 
         openib_btl->rd_num = mca_btl_openib_component.rd_num + log2(nprocs) * mca_btl_openib_component.srq_rd_per_peer; 
         if(openib_btl->rd_num > mca_btl_openib_component.srq_rd_max)
@@ -144,12 +232,14 @@ int mca_btl_openib_size_queues( struct mca_btl_openib_module_t* openib_btl, size
         if(!first_time) { 
             struct ibv_srq_attr srq_attr;
             srq_attr.max_wr = openib_btl->rd_num;
-            rc = ibv_modify_srq( openib_btl->srq_hp, &srq_attr, IBV_SRQ_MAX_WR); 
+            rc = ibv_modify_srq(openib_btl->srq[BTL_OPENIB_HP_QP],
+                    &srq_attr, IBV_SRQ_MAX_WR); 
             if(rc) { 
                 BTL_ERROR(("cannot resize high priority shared receive queue, error: %d", rc));
                 return OMPI_ERROR;
             }
-            rc = ibv_modify_srq(openib_btl->srq_lp, &srq_attr, IBV_SRQ_MAX_WR); 
+            rc = ibv_modify_srq(openib_btl->srq[BTL_OPENIB_LP_QP],
+                    &srq_attr, IBV_SRQ_MAX_WR); 
             if(rc) { 
                 BTL_ERROR(("cannot resize low priority shared receive queue, error: %d", rc));
                 return OMPI_ERROR;
@@ -158,7 +248,6 @@ int mca_btl_openib_size_queues( struct mca_btl_openib_module_t* openib_btl, size
         }
 	    
     } else 
-#endif
 	{
 	    min_cq_size = ( mca_btl_openib_component.rd_num >  (int32_t) mca_btl_openib_component.eager_rdma_num  ? 
 			    mca_btl_openib_component.rd_num : (int32_t) mca_btl_openib_component.eager_rdma_num ) * 
@@ -168,18 +257,20 @@ int mca_btl_openib_size_queues( struct mca_btl_openib_module_t* openib_btl, size
 
 
     if(min_cq_size > (int32_t) mca_btl_openib_component.ib_cq_size) { 
-
-        mca_btl_openib_component.ib_cq_size = min_cq_size > openib_btl->ib_dev_attr.max_cq ? 
-            openib_btl->ib_dev_attr.max_cq : min_cq_size;
+        mca_btl_openib_component.ib_cq_size = min_cq_size > (uint32_t)openib_btl->hca->ib_dev_attr.max_cqe ? 
+            (uint32_t)openib_btl->hca->ib_dev_attr.max_cqe : min_cq_size;
 #if OMPI_MCA_BTL_OPENIB_HAVE_RESIZE_CQ
         if(!first_time) { 
-            rc = ibv_resize_cq(openib_btl->ib_cq_lp, mca_btl_openib_component.ib_cq_size);
-            if(rc) {
+            rc = ibv_resize_cq(openib_btl->ib_cq[BTL_OPENIB_LP_QP], mca_btl_openib_component.ib_cq_size);
+            /* For ConnectX the resize CQ is not implemented and verbs returns -ENOSYS
+             * but should return ENOSYS. So it is reason for abs */
+            if(rc && ENOSYS != abs(rc)) {
                 BTL_ERROR(("cannot resize low priority completion queue, error: %d", rc));
                 return OMPI_ERROR;
             }
-            rc = ibv_resize_cq(openib_btl->ib_cq_hp, mca_btl_openib_component.ib_cq_size); 
-            if(rc) {
+            rc = ibv_resize_cq(openib_btl->ib_cq[BTL_OPENIB_HP_QP],
+                    mca_btl_openib_component.ib_cq_size);
+            if(rc && ENOSYS != abs(rc)) {
                 BTL_ERROR(("cannot resize high priority completion queue, error: %d", rc));
                 return OMPI_ERROR;
             }
@@ -189,14 +280,13 @@ int mca_btl_openib_size_queues( struct mca_btl_openib_module_t* openib_btl, size
     if(first_time) { 
         /* never been here before, setup cq and srq */
         mca_btl_openib_component.ib_cq_size = (int) mca_btl_openib_component.ib_cq_size > 
-            openib_btl->ib_dev_attr.max_cq ? 
-            openib_btl->ib_dev_attr.max_cq : 
+            (uint32_t)openib_btl->hca->ib_dev_attr.max_cqe ? 
+            (uint32_t)openib_btl->hca->ib_dev_attr.max_cqe : 
             (int) mca_btl_openib_component.ib_cq_size;
         return mca_btl_openib_create_cq_srq(openib_btl); 
     }
     return OMPI_SUCCESS;
 }
-
 /* 
  * delete the proc as reachable from this btl module 
  */
@@ -230,6 +320,19 @@ int mca_btl_openib_register(
 }
 
 
+/* 
+ *Register callback function for error handling..
+ */ 
+int mca_btl_openib_register_error_cb(
+                        struct mca_btl_base_module_t* btl, 
+                        mca_btl_base_module_error_cb_fn_t cbfunc)
+{
+    
+    mca_btl_openib_module_t* openib_btl = (mca_btl_openib_module_t*) btl; 
+    openib_btl->error_cb = cbfunc; /* stash for later */
+    return OMPI_SUCCESS;
+}
+
 /**
  * Allocate a segment.
  *
@@ -243,21 +346,20 @@ mca_btl_base_descriptor_t* mca_btl_openib_alloc(
     struct mca_btl_base_module_t* btl,
     size_t size)
 {
-    mca_btl_openib_frag_t* frag;
+    mca_btl_openib_frag_t* frag = NULL;
     mca_btl_openib_module_t* openib_btl; 
     int rc;
     openib_btl = (mca_btl_openib_module_t*) btl; 
     
     if(size <= mca_btl_openib_component.eager_limit){ 
-        MCA_BTL_IB_FRAG_ALLOC_EAGER(btl, frag, rc); 
-        frag->segment.seg_len = size; 
+        MCA_BTL_IB_FRAG_ALLOC_EAGER(btl, frag, rc);
     } else if(size <= mca_btl_openib_component.max_send_size) { 
         MCA_BTL_IB_FRAG_ALLOC_MAX(btl, frag, rc); 
-        frag->segment.seg_len = size;
-    } else { 
-        return NULL;
     }
     
+    if(NULL == frag)
+        return NULL;
+
     frag->segment.seg_len = size <= openib_btl->super.btl_eager_limit ? size : openib_btl->super.btl_eager_limit;  
     frag->base.des_flags = 0; 
     
@@ -276,10 +378,13 @@ int mca_btl_openib_free(
 {
     mca_btl_openib_frag_t* frag = (mca_btl_openib_frag_t*)des; 
 
-    if(frag->size == 0) {
-        btl->btl_mpool->mpool_release(btl->btl_mpool, 
-                                      (mca_mpool_base_registration_t*) 
-                                      frag->openib_reg); 
+    if(((MCA_BTL_OPENIB_SEND_FRAG_FRAG == frag->type) ||
+            (MCA_BTL_OPENIB_RECV_FRAG_FRAG == frag->type)) 
+            && frag->registration != NULL) {
+        btl->btl_mpool->mpool_deregister(btl->btl_mpool,
+                                      (mca_mpool_base_registration_t*)
+                                      frag->registration);
+        frag->registration = NULL;
     }
     MCA_BTL_IB_FRAG_RETURN(((mca_btl_openib_module_t*) btl), frag);
         
@@ -319,167 +424,101 @@ mca_btl_base_descriptor_t* mca_btl_openib_prepare_src(
     size_t* size
 )
 {
-    mca_btl_openib_module_t* openib_btl; 
-    mca_btl_openib_frag_t* frag; 
-    mca_mpool_openib_registration_t * openib_reg; 
-    struct iovec iov; 
-    uint32_t iov_count = 1; 
-    size_t max_data = *size; 
-    int32_t free_after; 
-    int rc; 
-    
-    openib_btl = (mca_btl_openib_module_t*) btl; 
-    openib_reg = (mca_mpool_openib_registration_t*) registration; 
+    mca_btl_openib_module_t *openib_btl;
+    mca_btl_openib_frag_t *frag = NULL;
+    mca_btl_openib_reg_t *openib_reg;
+    struct iovec iov;
+    uint32_t iov_count = 1;
+    size_t max_data = *size;
+    int rc;
 
-    
-    if(NULL != openib_reg &&  0 == ompi_convertor_need_buffers(convertor)){ 
-        size_t reg_len; 
+    openib_btl = (mca_btl_openib_module_t*)btl;
 
-       /* the memory is already pinned and we have contiguous user data */ 
+    if(ompi_convertor_need_buffers(convertor) == false && 0 == reserve) {
+        if(registration != NULL || max_data > btl->btl_max_send_size) {
+            MCA_BTL_IB_FRAG_ALLOC_SEND_FRAG(btl, frag, rc);
+            if(NULL == frag) {
+                return NULL;
+            }
 
-        MCA_BTL_IB_FRAG_ALLOC_FRAG(btl, frag, rc); 
-        if(NULL == frag){
-            return NULL; 
-        } 
+            iov.iov_len = max_data;
+            iov.iov_base = NULL;
         
-        iov.iov_len = max_data; 
-        iov.iov_base = NULL; 
-        
-        ompi_convertor_pack(convertor, &iov, &iov_count, &max_data, &free_after); 
+            ompi_convertor_pack(convertor, &iov, &iov_count, &max_data);
                 
-        frag->segment.seg_len = max_data; 
-        frag->segment.seg_addr.pval = iov.iov_base; 
-        
-        
-        reg_len = (unsigned char*)openib_reg->base_reg.bound - (unsigned char*)iov.iov_base + 1; 
-               
-        frag->mr = openib_reg->mr; 
-        frag->sg_entry.length = max_data; 
-        frag->sg_entry.lkey = frag->mr->lkey; 
-        
-        frag->sg_entry.addr = (unsigned long) iov.iov_base;
-        
-        frag->segment.seg_key.key32[0] = (uint32_t) frag->sg_entry.lkey; 
-        
-        frag->base.des_src = &frag->segment;
-        frag->base.des_src_cnt = 1;
-        frag->base.des_dst = NULL;
-        frag->base.des_dst_cnt = 0;
-        frag->base.des_flags = 0; 
-        frag->openib_reg= openib_reg; 
-        btl->btl_mpool->mpool_retain(btl->btl_mpool, (mca_mpool_base_registration_t*) openib_reg); 
-        return &frag->base;
-        
-    } else if( max_data > btl->btl_max_send_size && 
-               ompi_convertor_need_buffers(convertor) == 0 && 
-               reserve == 0) {
-        /* The user buffer is contigous and we are asked to send more than the max send size.  */            
-        
-        MCA_BTL_IB_FRAG_ALLOC_FRAG(openib_btl, frag, rc); 
-        if(NULL == frag){
-            return NULL; 
-        } 
-       
-        iov.iov_len = max_data; 
-        iov.iov_base = NULL; 
-        
-        ompi_convertor_pack(convertor, &iov, &iov_count, &max_data, &free_after); 
-        
-        frag->segment.seg_len = max_data; 
-        frag->segment.seg_addr.pval = iov.iov_base; 
-        frag->base.des_flags = 0; 
+            *size = max_data;
 
-        
-        rc = btl->btl_mpool->mpool_register(btl->btl_mpool,
-                                            iov.iov_base, 
-                                            max_data, 
-                                            0,
-                                            (mca_mpool_base_registration_t**) &openib_reg); 
-        if(OMPI_SUCCESS != rc || NULL == openib_reg) {
-            BTL_ERROR(("mpool_register(%p,%lu) failed", iov.iov_base, max_data));
-            MCA_BTL_IB_FRAG_RETURN(openib_btl, frag); 
+            if(NULL == registration) {
+                rc = btl->btl_mpool->mpool_register(btl->btl_mpool,
+                        iov.iov_base, max_data, 0, &registration);
+                if(OMPI_SUCCESS != rc || NULL == registration) {
+                    MCA_BTL_IB_FRAG_RETURN(openib_btl, frag);
+                    return NULL;
+                }
+                /* keep track of the registration we did */
+                frag->registration = (mca_btl_openib_reg_t*)registration;
+            }
+            openib_reg = (mca_btl_openib_reg_t*)registration;
+
+            frag->base.des_flags = 0;
+            frag->base.des_src = &frag->segment;
+            frag->base.des_src_cnt = 1;
+            frag->base.des_dst = NULL;
+            frag->base.des_dst_cnt = 0;
+            frag->base.des_flags = 0;
+
+            frag->sg_entry.length = max_data;
+            frag->sg_entry.lkey = openib_reg->mr->lkey;
+            frag->sg_entry.addr = (unsigned long)iov.iov_base;
+
+            frag->segment.seg_len = max_data;
+            frag->segment.seg_addr.pval = iov.iov_base;
+            frag->segment.seg_key.key32[0] = (uint32_t)frag->sg_entry.lkey;
+
+            BTL_VERBOSE(("frag->sg_entry.lkey = %lu .addr = %llu "
+                        "frag->segment.seg_key.key32[0] = %lu",
+                        frag->sg_entry.lkey, frag->sg_entry.addr,
+                        frag->segment.seg_key.key32[0]));
+
+            return &frag->base;
+        }
+    }
+
+    if(max_data + reserve <= btl->btl_eager_limit) {
+        /* the data is small enough to fit in the eager frag and
+         * memory is not prepinned */
+        MCA_BTL_IB_FRAG_ALLOC_EAGER(btl, frag, rc);
+    }
+
+    if(NULL == frag) {
+        /* the data doesn't fit into eager frag or eger frag is
+         * not available */
+        MCA_BTL_IB_FRAG_ALLOC_MAX(btl, frag, rc);
+        if(NULL == frag) {
             return NULL;
         }
-        
-        
-        frag->mr = openib_reg->mr; 
-        frag->sg_entry.length = max_data; 
-        frag->sg_entry.lkey = openib_reg->mr->lkey;
-        
-        frag->sg_entry.addr = (unsigned long) iov.iov_base;
-        
-        frag->segment.seg_key.key32[0] = (uint32_t) frag->mr->rkey; 
-            
-        frag->base.des_src = &frag->segment;
-        frag->base.des_src_cnt = 1;
-        frag->base.des_dst = NULL;
-        frag->base.des_dst_cnt = 0;
-        frag->openib_reg = openib_reg; 
-        BTL_VERBOSE(("frag->sg_entry.lkey = %lu .addr = %llu", frag->sg_entry.lkey, frag->sg_entry.addr)); 
-
-        return &frag->base;
-
-    } else if (max_data+reserve <=  btl->btl_eager_limit) { 
-        /* the data is small enough to fit in the eager frag and 
-           either we received no prepinned memory or leave pinned is 
-           not set
-        */    
-        MCA_BTL_IB_FRAG_ALLOC_EAGER(btl, frag, rc); 
-        if(NULL == frag) { 
-            return NULL; 
-        } 
-        
-        iov.iov_len = max_data; 
-        iov.iov_base = (unsigned char*)frag->segment.seg_addr.pval + reserve; 
-        
-        rc = ompi_convertor_pack(convertor, &iov, &iov_count, &max_data, &free_after); 
-        *size  = max_data; 
-        if( rc < 0 ) { 
-            MCA_BTL_IB_FRAG_RETURN(openib_btl, frag); 
-            return NULL; 
-        } 
-        
-        frag->segment.seg_len = max_data + reserve; 
-        frag->segment.seg_key.key32[0] = (uint32_t) frag->sg_entry.lkey; 
-        frag->base.des_src = &frag->segment;
-        frag->base.des_src_cnt = 1;
-        frag->base.des_dst = NULL;
-        frag->base.des_dst_cnt = 0;
-        frag->base.des_flags = 0; 
-        
-        return &frag->base; 
-        
-    } else {
-           
-        MCA_BTL_IB_FRAG_ALLOC_MAX(btl, frag, rc); 
-        if(NULL == frag) { 
-            return NULL; 
-        } 
-        if(max_data + reserve > btl->btl_max_send_size){ 
-            max_data = btl->btl_max_send_size - reserve; 
+        if(max_data + reserve > btl->btl_max_send_size) {
+            max_data = btl->btl_max_send_size - reserve;
         }
-        iov.iov_len = max_data; 
-        iov.iov_base = (unsigned char*)frag->segment.seg_addr.pval + reserve; 
-        
-        rc = ompi_convertor_pack(convertor, &iov, &iov_count, &max_data, &free_after); 
-        *size  = max_data; 
-
-        if( rc < 0 ) { 
-            MCA_BTL_IB_FRAG_RETURN(openib_btl, frag); 
-            return NULL; 
-        } 
-        
-        frag->segment.seg_len = max_data + reserve; 
-        frag->segment.seg_key.key32[0] = (uint32_t) frag->sg_entry.lkey; 
-        frag->base.des_src = &frag->segment;
-        frag->base.des_src_cnt = 1;
-        frag->base.des_dst = NULL;
-        frag->base.des_dst_cnt = 0;
-        frag->base.des_flags=0; 
-        
-        return &frag->base; 
     }
-    return NULL; 
+
+    iov.iov_len = max_data;
+    iov.iov_base = (unsigned char*)frag->segment.seg_addr.pval + reserve;
+    rc = ompi_convertor_pack(convertor, &iov, &iov_count, &max_data);
+    if(rc < 0) {
+        MCA_BTL_IB_FRAG_RETURN(openib_btl, frag);
+        return NULL;
+    }
+    *size  = max_data;
+    frag->segment.seg_len = max_data + reserve;
+    frag->segment.seg_key.key32[0] = (uint32_t)frag->sg_entry.lkey;
+    frag->base.des_src = &frag->segment;
+    frag->base.des_src_cnt = 1;
+    frag->base.des_dst = NULL;
+    frag->base.des_dst_cnt = 0;
+    frag->base.des_flags = 0;
+
+    return &frag->base;
 }
 
 /**
@@ -499,71 +538,62 @@ mca_btl_base_descriptor_t* mca_btl_openib_prepare_src(
 mca_btl_base_descriptor_t* mca_btl_openib_prepare_dst(
     struct mca_btl_base_module_t* btl,
     struct mca_btl_base_endpoint_t* endpoint,
-    mca_mpool_base_registration_t* registration, 
+    mca_mpool_base_registration_t* registration,
     struct ompi_convertor_t* convertor,
     size_t reserve,
     size_t* size)
 {
-    mca_btl_openib_module_t* openib_btl; 
-    mca_btl_openib_frag_t* frag; 
-    mca_mpool_openib_registration_t * openib_reg; 
-    int rc; 
-    long lb;
-    size_t reg_len; 
+    mca_btl_openib_module_t *openib_btl;
+    mca_btl_openib_frag_t *frag;
+    mca_btl_openib_reg_t *openib_reg;
+    int rc;
+    ptrdiff_t lb;
 
-    openib_btl = (mca_btl_openib_module_t*) btl; 
-    openib_reg = (mca_mpool_openib_registration_t*) registration; 
+    openib_btl = (mca_btl_openib_module_t*)btl;
     
-    MCA_BTL_IB_FRAG_ALLOC_FRAG(btl, frag, rc); 
-    if(NULL == frag){
-        return NULL; 
+    MCA_BTL_IB_FRAG_ALLOC_RECV_FRAG(btl, frag, rc);
+    if(NULL == frag) {
+        return NULL;
     }
     
     ompi_ddt_type_lb(convertor->pDesc, &lb);
-    frag->segment.seg_len = *size; 
-    frag->segment.seg_addr.pval = convertor->pBaseBuf + lb + convertor->bConverted; 
-    frag->base.des_flags = 0; 
+    frag->segment.seg_addr.pval = convertor->pBaseBuf + lb +
+        convertor->bConverted;
 
-    if(NULL!= openib_reg){ 
-        /* the memory is already pinned try to use it if the pinned region is large enough*/ 
-        reg_len = (unsigned char*)openib_reg->base_reg.bound - (unsigned char*)frag->segment.seg_addr.pval + 1; 
-        btl->btl_mpool->mpool_retain(btl->btl_mpool, 
-                                     (mca_mpool_base_registration_t*) openib_reg); 
-    }  else { 
-        /* we didn't get a memory registration passed in, so we have to register the region
-         * ourselves 
+    if(NULL == registration){
+        /* we didn't get a memory registration passed in, so we have to
+         * register the region ourselves
          */ 
-        
         rc = btl->btl_mpool->mpool_register(btl->btl_mpool,
-                                       frag->segment.seg_addr.pval,
-                                       *size, 
-                                       0,
-                                       (mca_mpool_base_registration_t**) &openib_reg);
-        if(OMPI_SUCCESS != rc || NULL == openib_reg) {
-            BTL_ERROR(("mpool_register(%p,%lu) failed: base %p lb %lu offset %lu", 
-                frag->segment.seg_addr.pval, *size, convertor->pBaseBuf, lb, convertor->bConverted));
+                frag->segment.seg_addr.pval, *size, 0, &registration);
+        if(OMPI_SUCCESS != rc || NULL == registration) {
             MCA_BTL_IB_FRAG_RETURN(openib_btl, frag);
             return NULL;
         }
+        /* keep track of the registration we did */
+        frag->registration = (mca_btl_openib_reg_t*)registration;
     }
+    openib_reg = (mca_btl_openib_reg_t*)registration;
 
-    
-    frag->mr = openib_reg->mr; 
-    frag->sg_entry.length = *size; 
-    frag->sg_entry.lkey = openib_reg->mr->lkey; 
-    frag->sg_entry.addr = (unsigned long) frag->segment.seg_addr.pval; 
-    
-    frag->segment.seg_key.key32[0] = frag->mr->rkey; 
-    
-    frag->base.des_dst = &frag->segment; 
-    frag->base.des_dst_cnt = 1; 
-    frag->base.des_src = NULL; 
-    frag->base.des_src_cnt = 0; 
-    frag->openib_reg = openib_reg; 
-    BTL_VERBOSE(("frag->sg_entry.lkey = %lu .addr = %llu frag->segment.seg_key.key32[0] = %lu" , frag->sg_entry.lkey, frag->sg_entry.addr, frag->segment.seg_key.key32[0])); 
+    frag->sg_entry.length = *size;
+    frag->sg_entry.lkey = openib_reg->mr->lkey;
+    frag->sg_entry.addr = (unsigned long) frag->segment.seg_addr.pval;
 
-    return &frag->base; 
-    
+    frag->segment.seg_len = *size;
+    frag->segment.seg_key.key32[0] = openib_reg->mr->rkey;
+
+    frag->base.des_dst = &frag->segment;
+    frag->base.des_dst_cnt = 1;
+    frag->base.des_src = NULL;
+    frag->base.des_src_cnt = 0;
+    frag->base.des_flags = 0;
+
+    BTL_VERBOSE(("frag->sg_entry.lkey = %lu .addr = %llu "
+                "frag->segment.seg_key.key32[0] = %lu",
+                frag->sg_entry.lkey, frag->sg_entry.addr,
+                frag->segment.seg_key.key32[0]));
+
+    return &frag->base;
 }
 
 int mca_btl_openib_finalize(struct mca_btl_base_module_t* btl)
@@ -646,11 +676,11 @@ int mca_btl_openib_put( mca_btl_base_module_t* btl,
     frag->wr_desc.sr_desc.opcode = IBV_WR_RDMA_WRITE; 
 
     /* check for a send wqe */
-    if (OPAL_THREAD_ADD32(&endpoint->sd_wqe_lp,-1) < 0) {
+    if (OPAL_THREAD_ADD32(&endpoint->sd_wqe[BTL_OPENIB_LP_QP],-1) < 0) {
 
-        OPAL_THREAD_ADD32(&endpoint->sd_wqe_lp,1);
+        OPAL_THREAD_ADD32(&endpoint->sd_wqe[BTL_OPENIB_LP_QP],1);
         OPAL_THREAD_LOCK(&endpoint->endpoint_lock);
-        opal_list_append(&endpoint->pending_frags_lp, (opal_list_item_t *)frag);
+        opal_list_append(&endpoint->pending_put_frags, (opal_list_item_t *)frag);
         OPAL_THREAD_UNLOCK(&endpoint->endpoint_lock);
         return OMPI_SUCCESS;
 
@@ -658,12 +688,12 @@ int mca_btl_openib_put( mca_btl_base_module_t* btl,
     } else {
         
         frag->wr_desc.sr_desc.send_flags = IBV_SEND_SIGNALED; 
-        frag->wr_desc.sr_desc.wr.rdma.remote_addr = (unsigned long) frag->base.des_dst->seg_addr.pval; 
+        frag->wr_desc.sr_desc.wr.rdma.remote_addr = frag->base.des_dst->seg_addr.lval; 
         frag->wr_desc.sr_desc.wr.rdma.rkey = frag->base.des_dst->seg_key.key32[0]; 
         frag->sg_entry.addr = (unsigned long) frag->base.des_src->seg_addr.pval; 
         frag->sg_entry.length  = frag->base.des_src->seg_len; 
         
-        if(ibv_post_send(endpoint->lcl_qp_lp, 
+        if(ibv_post_send(endpoint->lcl_qp[BTL_OPENIB_LP_QP], 
                          &frag->wr_desc.sr_desc, 
                          &bad_wr)){ 
             rc = OMPI_ERROR;
@@ -671,17 +701,13 @@ int mca_btl_openib_put( mca_btl_base_module_t* btl,
             rc = OMPI_SUCCESS;
         }
     
-#ifdef OMPI_MCA_BTL_OPENIB_HAVE_SRQ 
         if(mca_btl_openib_component.use_srq) { 
-            MCA_BTL_OPENIB_POST_SRR_HIGH(openib_btl, 1); 
-            MCA_BTL_OPENIB_POST_SRR_LOW(openib_btl, 1);         
+            mca_btl_openib_post_srr(openib_btl, 1, BTL_OPENIB_HP_QP);
+            mca_btl_openib_post_srr(openib_btl, 1, BTL_OPENIB_LP_QP);
         } else { 
-#endif 
-            MCA_BTL_OPENIB_ENDPOINT_POST_RR_HIGH(endpoint, 1); 
-            MCA_BTL_OPENIB_ENDPOINT_POST_RR_LOW(endpoint, 1); 
-#ifdef OMPI_MCA_BTL_OPENIB_HAVE_SRQ
+            btl_openib_endpoint_post_rr(endpoint, 1, BTL_OPENIB_HP_QP);
+            btl_openib_endpoint_post_rr(endpoint, 1, BTL_OPENIB_LP_QP);
         }
-#endif 
     }
     return rc; 
 }
@@ -703,33 +729,33 @@ int mca_btl_openib_get( mca_btl_base_module_t* btl,
     frag->wr_desc.sr_desc.opcode = IBV_WR_RDMA_READ; 
 
     /* check for a send wqe */
-    if (OPAL_THREAD_ADD32(&endpoint->sd_wqe_lp,-1) < 0) {
+    if (OPAL_THREAD_ADD32(&endpoint->sd_wqe[BTL_OPENIB_LP_QP],-1) < 0) {
 
-        OPAL_THREAD_ADD32(&endpoint->sd_wqe_lp,1);
+        OPAL_THREAD_ADD32(&endpoint->sd_wqe[BTL_OPENIB_LP_QP],1);
         OPAL_THREAD_LOCK(&endpoint->endpoint_lock);
-        opal_list_append(&endpoint->pending_frags_lp, (opal_list_item_t *)frag);
+        opal_list_append(&endpoint->pending_get_frags, (opal_list_item_t*)frag);
         OPAL_THREAD_UNLOCK(&endpoint->endpoint_lock);
         return OMPI_SUCCESS;
 
     /* check for a get token */
     } else if(OPAL_THREAD_ADD32(&endpoint->get_tokens,-1) < 0) {
 
-        OPAL_THREAD_ADD32(&endpoint->sd_wqe_lp,1);
+        OPAL_THREAD_ADD32(&endpoint->sd_wqe[BTL_OPENIB_LP_QP],1);
         OPAL_THREAD_ADD32(&endpoint->get_tokens,1);
         OPAL_THREAD_LOCK(&endpoint->endpoint_lock);
-        opal_list_append(&endpoint->pending_frags_lp, (opal_list_item_t*)frag);
+        opal_list_append(&endpoint->pending_get_frags, (opal_list_item_t*)frag);
         OPAL_THREAD_UNLOCK(&endpoint->endpoint_lock);
         return OMPI_SUCCESS;
 
     } else { 
     
         frag->wr_desc.sr_desc.send_flags = IBV_SEND_SIGNALED; 
-        frag->wr_desc.sr_desc.wr.rdma.remote_addr = (unsigned long) frag->base.des_src->seg_addr.pval; 
+        frag->wr_desc.sr_desc.wr.rdma.remote_addr = frag->base.des_src->seg_addr.lval; 
         frag->wr_desc.sr_desc.wr.rdma.rkey = frag->base.des_src->seg_key.key32[0]; 
         frag->sg_entry.addr = (unsigned long) frag->base.des_dst->seg_addr.pval; 
         frag->sg_entry.length  = frag->base.des_dst->seg_len; 
         
-        if(ibv_post_send(endpoint->lcl_qp_lp, 
+        if(ibv_post_send(endpoint->lcl_qp[BTL_OPENIB_LP_QP], 
                          &frag->wr_desc.sr_desc, 
                          &bad_wr)){ 
             BTL_ERROR(("error posting send request errno (%d) says %s", errno, strerror(errno))); 
@@ -738,17 +764,13 @@ int mca_btl_openib_get( mca_btl_base_module_t* btl,
             rc = ORTE_SUCCESS;
         }
         
-#ifdef OMPI_MCA_BTL_OPENIB_HAVE_SRQ 
         if(mca_btl_openib_component.use_srq) { 
-            MCA_BTL_OPENIB_POST_SRR_HIGH(openib_btl, 1); 
-            MCA_BTL_OPENIB_POST_SRR_LOW(openib_btl, 1);         
+            mca_btl_openib_post_srr(openib_btl, 1, BTL_OPENIB_HP_QP);
+            mca_btl_openib_post_srr(openib_btl, 1, BTL_OPENIB_LP_QP);
         } else { 
-#endif 
-            MCA_BTL_OPENIB_ENDPOINT_POST_RR_HIGH(endpoint, 1); 
-            MCA_BTL_OPENIB_ENDPOINT_POST_RR_LOW(endpoint, 1); 
-#ifdef OMPI_MCA_BTL_OPENIB_HAVE_SRQ
+            btl_openib_endpoint_post_rr(endpoint, 1, BTL_OPENIB_HP_QP);
+            btl_openib_endpoint_post_rr(endpoint, 1, BTL_OPENIB_LP_QP);
         }
-#endif 
     }
     return rc; 
 }
@@ -762,70 +784,69 @@ int mca_btl_openib_create_cq_srq(mca_btl_openib_module_t *openib_btl)
     /* Allocate Protection Domain */ 
     openib_btl->poll_cq = false; 
     
-#ifdef OMPI_MCA_BTL_OPENIB_HAVE_SRQ
-    if(mca_btl_openib_component.use_srq) { 
+    if (mca_btl_openib_component.use_srq) { 
         
         struct ibv_srq_init_attr attr; 
         attr.attr.max_wr = mca_btl_openib_component.srq_rd_max;
         attr.attr.max_sge = mca_btl_openib_component.ib_sg_list_size;
 
-        openib_btl->srd_posted_hp = 0; 
-        openib_btl->srd_posted_lp = 0; 
+        openib_btl->srd_posted[BTL_OPENIB_HP_QP] = 0; 
+        openib_btl->srd_posted[BTL_OPENIB_LP_QP] = 0; 
         
-        openib_btl->srq_hp = ibv_create_srq(openib_btl->ib_pd, &attr); 
-        if(NULL == openib_btl->srq_hp) { 
-            BTL_ERROR(("error in ibv_create_srq\n")); 
+        openib_btl->srq[BTL_OPENIB_HP_QP] =
+            ibv_create_srq(openib_btl->hca->ib_pd, &attr); 
+        if (NULL == openib_btl->srq[BTL_OPENIB_HP_QP]) { 
+            show_init_error(__FILE__, __LINE__, "ibv_create_srq",
+                            ibv_get_device_name(openib_btl->hca->ib_dev));
             return OMPI_ERROR; 
         }
         
-        openib_btl->srq_lp = ibv_create_srq(openib_btl->ib_pd, &attr); 
-        if(NULL == openib_btl->srq_hp) { 
-            BTL_ERROR(("error in ibv_create_srq\n")); 
+        openib_btl->srq[BTL_OPENIB_LP_QP] =
+            ibv_create_srq(openib_btl->hca->ib_pd, &attr); 
+        if (NULL == openib_btl->srq[BTL_OPENIB_LP_QP]) { 
+            show_init_error(__FILE__, __LINE__, "ibv_create_srq",
+                            ibv_get_device_name(openib_btl->hca->ib_dev));
             return OMPI_ERROR; 
         }
         
         
     } else { 
-        openib_btl->srq_hp = NULL; 
-        openib_btl->srq_lp = NULL;
+        openib_btl->srq[BTL_OPENIB_HP_QP] = NULL; 
+        openib_btl->srq[BTL_OPENIB_LP_QP] = NULL;
     } 
-#endif
     
     /* Create the low and high priority queue pairs */ 
 #if OMPI_MCA_BTL_OPENIB_IBV_CREATE_CQ_ARGS == 3
-    openib_btl->ib_cq_lp = 
-        ibv_create_cq(openib_btl->ib_dev_context,
+    openib_btl->ib_cq[BTL_OPENIB_LP_QP] =
+        ibv_create_cq(openib_btl->hca->ib_dev_context,
                 mca_btl_openib_component.ib_cq_size, NULL); 
 #else
-    openib_btl->ib_cq_lp = 
-        ibv_create_cq(openib_btl->ib_dev_context,
+    openib_btl->ib_cq[BTL_OPENIB_LP_QP] =
+        ibv_create_cq(openib_btl->hca->ib_dev_context,
                 mca_btl_openib_component.ib_cq_size, NULL, NULL, 0); 
 #endif
     
-    if(NULL == openib_btl->ib_cq_lp) {
-        BTL_ERROR(("error creating low priority cq for %s errno says %s\n",
-                  ibv_get_device_name(openib_btl->ib_dev), 
-                  strerror(errno))); 
+    if (NULL == openib_btl->ib_cq[BTL_OPENIB_LP_QP]) {
+        show_init_error(__FILE__, __LINE__, "ibv_create_cq",
+                        ibv_get_device_name(openib_btl->hca->ib_dev));
         return OMPI_ERROR;
     }
 
 #if OMPI_MCA_BTL_OPENIB_IBV_CREATE_CQ_ARGS == 3
-    openib_btl->ib_cq_hp = 
-        ibv_create_cq(openib_btl->ib_dev_context,
+    openib_btl->ib_cq[BTL_OPENIB_HP_QP] =
+        ibv_create_cq(openib_btl->hca->ib_dev_context,
                 mca_btl_openib_component.ib_cq_size, NULL); 
 #else
-    openib_btl->ib_cq_hp = 
-        ibv_create_cq(openib_btl->ib_dev_context,
+    openib_btl->ib_cq[BTL_OPENIB_HP_QP] =
+        ibv_create_cq(openib_btl->hca->ib_dev_context,
                 mca_btl_openib_component.ib_cq_size, NULL, NULL, 0); 
 #endif    
 
-    if(NULL == openib_btl->ib_cq_hp) {
-        BTL_ERROR(("error creating high priority cq for %s errno says %s\n", 
-                  ibv_get_device_name(openib_btl->ib_dev), 
-                  strerror(errno))); 
+    if(NULL == openib_btl->ib_cq[BTL_OPENIB_HP_QP]) {
+        show_init_error(__FILE__, __LINE__, "ibv_create_cq",
+                        ibv_get_device_name(openib_btl->hca->ib_dev));
         return OMPI_ERROR;
     }
-        
     
     return OMPI_SUCCESS;
 }
