@@ -1,4 +1,3 @@
-
 /*
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
@@ -10,15 +9,18 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2006-2007 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2006      Los Alamos National Security, LLC.  All rights
+ *                         reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
  * 
  * $HEADER$
- */
-/**
+ *
  * @file
  */
+
 #ifndef MCA_PTL_IB_H
 #define MCA_PTL_IB_H
 
@@ -40,20 +42,23 @@
 
 #include "ompi/mca/btl/btl.h"
 #include "ompi/mca/btl/base/base.h" 
-#include "btl_openib_endpoint.h"
+
+#include "btl_openib_frag.h"
 
 #if defined(c_plusplus) || defined(__cplusplus)
 extern "C" {
 #endif
 
 #define MCA_BTL_IB_LEAVE_PINNED 1
+#define IB_DEFAULT_GID_PREFIX 0xfe80000000000000ll
+#define MCA_BTL_IB_PKEY_MASK 0x7fff
 
 /**
  * Infiniband (IB) BTL component.
  */
 
 struct mca_btl_openib_component_t {
-    mca_btl_base_component_1_0_0_t          super;  /**< base BTL component */ 
+    mca_btl_base_component_1_0_1_t          super;  /**< base BTL component */ 
 
     int                                ib_max_btls;
     /**< maximum number of hcas available to the IB component */
@@ -105,6 +110,7 @@ struct mca_btl_openib_component_t {
     uint32_t ib_cq_size;   /**< Max outstanding CQE on the CQ */  
     uint32_t ib_sg_list_size; /**< Max scatter/gather descriptor entries on the WQ*/ 
     uint32_t ib_pkey_ix; 
+    uint32_t ib_pkey_val;
     uint32_t ib_psn; 
     uint32_t ib_qp_ous_rd_atom; 
     uint32_t ib_mtu; 
@@ -115,20 +121,69 @@ struct mca_btl_openib_component_t {
     uint32_t ib_max_rdma_dst_ops; 
     uint32_t ib_service_level; 
     uint32_t ib_static_rate; 
-    uint32_t ib_src_path_bits; 
     uint32_t use_eager_rdma;
-    uint32_t eager_rdma_threshold;
+    int32_t eager_rdma_threshold;
     uint32_t eager_rdma_num;
-    uint32_t max_eager_rdma;
+    int32_t max_eager_rdma;
+    uint32_t btls_per_lid;
+    uint32_t max_lmc;
+    uint32_t buffer_alignment;
 
+    /** Colon-delimited list of filenames for HCA parameters */
+    char *hca_params_file_names;
+
+    /** Whether we're in verbose mode or not */
+    bool verbose;
+
+    /** Whether we want a warning if no HCA-specific parameters are
+        found in INI files */
+    bool warn_no_hca_params_found;
+    /** Whether we want a warning if non default GID prefix is not configured
+        on multiport setup */
+    bool warn_default_gid_prefix;
+#ifdef HAVE_IBV_FORK_INIT
+    /** Whether we want fork support or not */
+    int want_fork_support;
+#endif
 }; typedef struct mca_btl_openib_component_t mca_btl_openib_component_t;
 
-extern mca_btl_openib_component_t mca_btl_openib_component;
+OMPI_MODULE_DECLSPEC extern mca_btl_openib_component_t mca_btl_openib_component;
 
 typedef mca_btl_base_recv_reg_t mca_btl_openib_recv_reg_t; 
     
+struct mca_btl_openib_port_info_t {
+    uint32_t mtu;
+#if OMPI_ENABLE_HETEROGENEOUS_SUPPORT 
+    uint8_t padding[4]; 
+#endif
+    uint64_t subnet_id;
+};
+typedef struct mca_btl_openib_port_info_t mca_btl_openib_port_info_t;
 
+#define MCA_BTL_OPENIB_PORT_INFO_NTOH(hdr)     \
+    do {                              \
+        (hdr).mtu = ntohl((hdr).mtu); \
+        (hdr).subnet_id = ntoh64((hdr).subnet_id); \
+    } while (0)
+#define MCA_BTL_OPENIB_PORT_INFO_HTON(hdr)     \
+    do {                              \
+        (hdr).mtu = htonl((hdr).mtu); \
+        (hdr).subnet_id = hton64((hdr).subnet_id); \
+    } while (0)
 
+struct mca_btl_openib_hca_t {
+    struct ibv_device *ib_dev;  /* the ib device */
+    struct ibv_context *ib_dev_context;
+    struct ibv_device_attr ib_dev_attr;
+    struct ibv_pd *ib_pd;
+    mca_mpool_base_module_t *mpool;
+    /* MTU for this HCA */
+    uint32_t mtu;
+    /* Whether this HCA supports eager RDMA */
+    uint8_t use_eager_rdma;
+    uint8_t btls;              /** < number of btls using this HCA */
+};
+typedef struct mca_btl_openib_hca_t mca_btl_openib_hca_t;
 /**
  * IB PTL Interface
  */
@@ -136,16 +191,14 @@ struct mca_btl_openib_module_t {
     mca_btl_base_module_t  super;  /**< base PTL interface */
     bool btl_inited; 
     mca_btl_openib_recv_reg_t ib_reg[256]; 
-    mca_btl_openib_port_info_t port_info;  /* contains only the subnet right now */ 
+    mca_btl_openib_port_info_t port_info;  /* contains only the subnet id right now */ 
+    mca_btl_openib_hca_t *hca;
     uint8_t port_num;           /**< ID of the PORT */ 
-    struct ibv_device *ib_dev;  /* the ib device */
-    struct ibv_device_attr ib_dev_attr;
-    struct ibv_context *ib_dev_context; 
-    struct ibv_pd *ib_pd; 
-    struct ibv_cq *ib_cq_hp;
-    struct ibv_cq *ib_cq_lp; 
-    struct ibv_port_attr* ib_port_attr; 
-    struct ibv_recv_wr* rd_desc_post;
+    uint16_t pkey_index;
+    struct ibv_cq *ib_cq[2];
+    struct ibv_port_attr ib_port_attr; 
+    uint16_t lid;                      /**< lid that is actually used (for LMC) */
+    uint8_t src_path_bits;             /**< offset from base lid (for LMC) */
 
     ompi_free_list_t send_free_eager;  /**< free list of eager buffer descriptors */
     ompi_free_list_t send_free_max;    /**< free list of max buffer descriptors */ 
@@ -153,9 +206,9 @@ struct mca_btl_openib_module_t {
     
     ompi_free_list_t recv_free_eager;  /**< High priority free list of buffer descriptors */
     ompi_free_list_t recv_free_max;    /**< Low priority free list of buffer descriptors */ 
+    ompi_free_list_t recv_free_frag;   /**< free list of frags only... used for pining memory */ 
 
-    opal_list_t reg_mru_list;      /**< a most recently used list of mca_mpool_openib_registration_t 
-                                       entries, this allows us to keep a working set of memory pinned */ 
+    ompi_free_list_t send_free_control; /**< frags for control massages */ 
     opal_mutex_t ib_lock;          /**< module level lock */ 
     
     
@@ -163,74 +216,33 @@ struct mca_btl_openib_module_t {
     size_t ib_inline_max; /**< max size of inline send*/ 
     bool poll_cq; 
     
-    
-#ifdef OMPI_MCA_BTL_OPENIB_HAVE_SRQ
-    struct ibv_srq *srq_hp; 
-    struct ibv_srq *srq_lp;
-    int32_t srd_posted_hp; 
-    int32_t srd_posted_lp; 
-#endif 
+    struct ibv_srq *srq[2]; 
+    int32_t srd_posted[2]; 
     int32_t num_peers;
     int32_t  rd_num; 
     int32_t  rd_low;
     
-    int32_t sd_tokens_hp; 
-    /**< number of high priority frags that  can be outstanding (down counter) */ 
-    int32_t sd_tokens_lp; 
-    /**< number of low priority frags that  can be outstanding (down counter) */ 
+    int32_t sd_tokens[2];
+    /**< number of frags that  can be outstanding (down counter) */ 
     
-    opal_list_t                 pending_frags_hp; 
-    /**< list of pending high priority frags */ 
+    opal_list_t pending_frags[2]; /**< list of pending frags */ 
 
-    opal_list_t                 pending_frags_lp; 
-    /**< list of pending low priority frags */ 
-
-    opal_mutex_t eager_rdma_lock;
     size_t eager_rdma_frag_size; /**< length of eager frag */
     orte_pointer_array_t *eager_rdma_buffers; /**< RDMA buffers to poll */
-    uint32_t eager_rdma_buffers_count; /**< number of RDMA buffers */
+    volatile int32_t eager_rdma_buffers_count; /**< number of RDMA buffers */
+
+    mca_btl_base_module_error_cb_fn_t error_cb; /**< error handler */
+   
+    orte_pointer_array_t *endpoints;
 }; typedef struct mca_btl_openib_module_t mca_btl_openib_module_t;
-    
-struct mca_btl_openib_frag_t; 
+
 extern mca_btl_openib_module_t mca_btl_openib_module;
 
-/**
- * Register IB component parameters with the MCA framework
- */
-extern int mca_btl_openib_component_open(void);
-
-/**
- * Any final cleanup before being unloaded.
- */
-extern int mca_btl_openib_component_close(void);
-
-/**
- * IB component initialization.
- * 
- * @param num_btl_modules (OUT)                  Number of BTLs returned in BTL array.
- * @param allow_multi_user_threads (OUT)  Flag indicating wether BTL supports user threads (TRUE)
- * @param have_hidden_threads (OUT)       Flag indicating wether BTL uses threads (TRUE)
- *
- *  (1) read interface list from kernel and compare against component parameters
- *      then create a BTL instance for selected interfaces
- *  (2) setup IB listen socket for incoming connection attempts
- *  (3) publish BTL addressing info 
- *
- */
-extern mca_btl_base_module_t** mca_btl_openib_component_init(
-    int *num_btl_modules, 
-    bool allow_multi_user_threads,
-    bool have_hidden_threads
-);
-
-
-/**
- * IB component progress.
- */
-extern int mca_btl_openib_component_progress(
-                                             void
-                                             );
-
+struct mca_btl_openib_reg_t {
+    mca_mpool_base_registration_t base;
+    struct ibv_mr *mr;
+};
+typedef struct mca_btl_openib_reg_t mca_btl_openib_reg_t;
 
 /**
  * Register a callback function that is called on receipt
@@ -250,6 +262,19 @@ int mca_btl_openib_register(
     mca_btl_base_module_recv_cb_fn_t cbfunc,
     void* cbdata
 );
+
+
+/**
+ * Register a callback function that is called on error..
+ *
+ * @param btl (IN)     BTL module
+ * @return             Status indicating if cleanup was successful
+ */
+
+int mca_btl_openib_register_error_cb(
+    struct mca_btl_base_module_t* btl,
+    mca_btl_base_module_error_cb_fn_t cbfunc
+);
                                                                                                                      
 
 /**
@@ -410,81 +435,51 @@ extern mca_btl_base_descriptor_t* mca_btl_openib_prepare_dst(
  * @param frag (IN)  IB send fragment
  *
  */
-extern void mca_btl_openib_send_frag_return(
-                                            struct mca_btl_base_module_t* btl,
-                                            struct mca_btl_openib_frag_t*
-                                            );
+extern void mca_btl_openib_send_frag_return(mca_btl_base_module_t* btl,
+        mca_btl_openib_frag_t*);
 
 
+int mca_btl_openib_create_cq_srq(mca_btl_openib_module_t* openib_btl); 
 
-#ifdef OMPI_MCA_BTL_OPENIB_HAVE_SRQ
+#define BTL_OPENIB_HP_QP 0
+#define BTL_OPENIB_LP_QP 1
 
-#define MCA_BTL_OPENIB_POST_SRR_HIGH(openib_btl, additional) \
-{ \
-    do{ \
-        OPAL_THREAD_LOCK(&openib_btl->ib_lock); \
-        if(openib_btl->srd_posted_hp <= openib_btl->rd_low+additional && \
-           openib_btl->srd_posted_hp < openib_btl->rd_num){ \
-           MCA_BTL_OPENIB_POST_SRR_SUB(openib_btl->rd_num -  \
-                                      openib_btl->srd_posted_hp, \
-                                      openib_btl, \
-                                      &openib_btl->recv_free_eager, \
-                                      &openib_btl->srd_posted_hp, \
-                                      openib_btl->srq_hp); \
-        } \
-        OPAL_THREAD_UNLOCK(&openib_btl->ib_lock); \
-    } while(0); \
+static inline int mca_btl_openib_post_srr(mca_btl_openib_module_t* openib_btl,
+        const int additional, const int prio)
+{
+    OPAL_THREAD_LOCK(&openib_btl->ib_lock);
+    if(openib_btl->srd_posted[prio] <= openib_btl->rd_low + additional &&
+             openib_btl->srd_posted[prio] < openib_btl->rd_num) {
+        int32_t i, rc;
+        int32_t num_post = openib_btl->rd_num - openib_btl->srd_posted[prio];
+        ompi_free_list_item_t* item;
+        mca_btl_openib_frag_t* frag;
+        struct ibv_recv_wr *bad_wr;
+        ompi_free_list_t *free_list;
+
+        if(BTL_OPENIB_HP_QP == prio)
+            free_list = &openib_btl->recv_free_eager;
+        else
+            free_list = &openib_btl->recv_free_max;
+
+        for(i = 0; i < num_post; i++) {
+            OMPI_FREE_LIST_WAIT(free_list, item, rc);
+            frag = (mca_btl_openib_frag_t*)item;
+            if(ibv_post_srq_recv(openib_btl->srq[prio], &frag->wr_desc.rd_desc,
+                        &bad_wr)) {
+                BTL_ERROR(("error posting receive descriptors to shared "
+                            "receive queue: %s", strerror(errno)));
+                OPAL_THREAD_UNLOCK(&openib_btl->ib_lock);
+                return OMPI_ERROR;
+            }
+        }
+        OPAL_THREAD_ADD32(&openib_btl->srd_posted[prio], num_post);
+    }
+    OPAL_THREAD_UNLOCK(&openib_btl->ib_lock);
+
+    return OMPI_SUCCESS;
 }
 
-#define MCA_BTL_OPENIB_POST_SRR_LOW(openib_btl, additional) \
-{ \
-    do { \
-    OPAL_THREAD_LOCK(&openib_btl->ib_lock); \
-    if(openib_btl->srd_posted_lp <= openib_btl->rd_low+additional && \
-       openib_btl->srd_posted_lp < openib_btl->rd_num){ \
-        MCA_BTL_OPENIB_POST_SRR_SUB(openib_btl->rd_num -  \
-                                            openib_btl->srd_posted_lp, \
-                                            openib_btl, \
-                                            &openib_btl->recv_free_max, \
-                                            &openib_btl->srd_posted_lp, \
-                                            openib_btl->srq_lp); \
-    } \
-    OPAL_THREAD_UNLOCK(&openib_btl->ib_lock); \
-    } while(0); \
-}
-
-
-#define MCA_BTL_OPENIB_POST_SRR_SUB(cnt, \
-                                   openib_btl, \
-                                   frag_list, \
-                                   srd_posted, \
-                                   srq) \
-{\
-    do { \
-    int32_t i; \
-    int32_t num_post = cnt;                          \
-    ompi_free_list_item_t* item = NULL;              \
-    mca_btl_openib_frag_t* frag = NULL;              \
-    struct ibv_recv_wr *bad_wr; \
-    int32_t rc; \
-    for(i = 0; i < num_post; i++) { \
-        OMPI_FREE_LIST_WAIT(frag_list, item, rc); \
-        frag = (mca_btl_openib_frag_t*) item; \
-        frag->sg_entry.length = frag->size + \
-            ((unsigned char*) frag->segment.seg_addr.pval-  \
-             (unsigned char*) frag->hdr);  \
-       if(ibv_post_srq_recv(srq, &frag->wr_desc.rd_desc, &bad_wr)) { \
-           BTL_ERROR(("error posting receive descriptors to shared receive queue: %s",\
-                   strerror(errno))); \
-           return OMPI_ERROR; \
-       }\
-    }\
-    OPAL_THREAD_ADD32(srd_posted,  num_post); \
-   } while(0);\
-}
-
-#endif
-
 #if defined(c_plusplus) || defined(__cplusplus)
 }
 #endif
