@@ -1,15 +1,17 @@
 /*
- * Copyright (c) 2004-2006 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2006      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2006-2007 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2007      Los Alamos National Security, LLC.  All rights
+ *                         reserved. 
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -24,6 +26,8 @@
  */
 
 #include "orte_config.h"
+#include "orte/orte_constants.h"
+#include "orte/orte_types.h"
 
 #include <sys/types.h>
 #ifdef HAVE_UNISTD_H
@@ -36,6 +40,9 @@
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
+#ifdef HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
@@ -43,7 +50,7 @@
 #include <fcntl.h>
 #endif
 
-#include "opal/install_dirs.h"
+#include "opal/mca/installdirs/installdirs.h"
 #include "opal/util/argv.h"
 #include "opal/util/output.h"
 #include "opal/util/opal_environ.h"
@@ -51,28 +58,31 @@
 #include "opal/util/show_help.h"
 #include "opal/util/basename.h"
 #include "opal/mca/base/mca_base_param.h"
+
 #include "orte/runtime/runtime.h"
-#include "orte/orte_constants.h"
-#include "orte/orte_types.h"
-#include "orte/orte_constants.h"
-#include "orte/mca/pls/pls.h"
-#include "orte/mca/pls/base/base.h"
 #include "orte/mca/ns/base/base.h"
 #include "orte/mca/rml/rml.h"
 #include "orte/mca/errmgr/errmgr.h"
-#include "orte/mca/rmaps/base/rmaps_base_map.h"
+#include "orte/mca/smr/smr.h"
+#include "orte/mca/rmaps/rmaps.h"
+
+#include "orte/mca/pls/pls.h"
+#include "orte/mca/pls/base/base.h"
+#include "orte/mca/pls/base/pls_private.h"
 #include "pls_slurm.h"
 
 
 /*
  * Local functions
  */
-static int pls_slurm_launch(orte_jobid_t jobid);
-static int pls_slurm_terminate_job(orte_jobid_t jobid);
+static int pls_slurm_launch_job(orte_jobid_t jobid);
+static int pls_slurm_terminate_job(orte_jobid_t jobid, struct timeval *timeout, opal_list_t *attrs);
+static int pls_slurm_terminate_orteds(orte_jobid_t jobid, struct timeval *timeout, opal_list_t *attrs);
 static int pls_slurm_terminate_proc(const orte_process_name_t *name);
-static int pls_slurm_signal_job(orte_jobid_t jobid, int32_t signal);
+static int pls_slurm_signal_job(orte_jobid_t jobid, int32_t signal, opal_list_t *attrs);
 static int pls_slurm_signal_proc(const orte_process_name_t *name, int32_t signal);
 static int pls_slurm_finalize(void);
+static int pls_slurm_cancel_operation(void);
 
 static int pls_slurm_start_proc(int argc, char **argv, char **env,
                                 char *prefix);
@@ -81,12 +91,14 @@ static int pls_slurm_start_proc(int argc, char **argv, char **env,
 /*
  * Global variable
  */
-orte_pls_base_module_1_0_0_t orte_pls_slurm_module = {
-    pls_slurm_launch,
+orte_pls_base_module_1_3_0_t orte_pls_slurm_module = {
+    pls_slurm_launch_job,
     pls_slurm_terminate_job,
+    pls_slurm_terminate_orteds,
     pls_slurm_terminate_proc,
     pls_slurm_signal_job,
     pls_slurm_signal_proc,
+    pls_slurm_cancel_operation,
     pls_slurm_finalize
 };
 
@@ -96,25 +108,20 @@ orte_pls_base_module_1_0_0_t orte_pls_slurm_module = {
 static pid_t srun_pid = 0;
 
 
-/*
- * External
- */
-extern char **environ;
-
-
-static int pls_slurm_launch(orte_jobid_t jobid)
+static int pls_slurm_launch_job(orte_jobid_t jobid)
 {
-    opal_list_t nodes, mapping_list;
-    opal_list_item_t *item, *item2;
+    orte_job_map_t *map;
+    opal_list_item_t *item;
     size_t num_nodes;
     orte_vpid_t vpid;
-    char *jobid_string;
+    orte_vpid_t start_vpid;
+    char *jobid_string = NULL;
     char *uri, *param;
     char **argv;
     int argc;
     int rc;
     char *tmp;
-    char** env;
+    char** env = NULL;
     char* var;
     char *nodelist_flat;
     char **nodelist_argv;
@@ -124,32 +131,70 @@ static int pls_slurm_launch(orte_jobid_t jobid)
     char **custom_strings;
     int num_args, i;
     char *cur_prefix;
+    opal_list_t daemons;
+    orte_pls_daemon_info_t *dmn;
+    struct timeval joblaunchstart, launchstart, launchstop;
 
-    /* Query the list of nodes allocated and mapped to this job.
+    if (mca_pls_slurm_component.timing) {
+        if (0 != gettimeofday(&joblaunchstart, NULL)) {
+            opal_output(0, "pls_slurm: could not obtain job start time");
+        }        
+    }
+    
+    /* setup a list that will contain the info for all the daemons
+     * so we can store it on the registry when done
+     */
+    OBJ_CONSTRUCT(&daemons, opal_list_t);
+    
+    /* Query the map for this job.
      * We need the entire mapping for a couple of reasons:
      *  - need the prefix to start with.
      *  - need to know if we are launching on a subset of the allocated nodes
      * All other mapping responsibilities fall to orted in the fork PLS
      */
-    OBJ_CONSTRUCT(&nodes, opal_list_t);
-    OBJ_CONSTRUCT(&mapping_list, opal_list_t);
-    rc = orte_rmaps_base_mapped_node_query(&mapping_list, &nodes, jobid);
+    rc = orte_rmaps.get_job_map(&map, jobid);
     if (ORTE_SUCCESS != rc) {
-        goto cleanup;
+        ORTE_ERROR_LOG(rc);
+        OBJ_DESTRUCT(&daemons);
+        return rc;
     }
 
+    /* if the user requested that we re-use daemons,
+     * launch the procs on any existing, re-usable daemons
+     */
+    if (orte_pls_base.reuse_daemons) {
+        if (ORTE_SUCCESS != (rc = orte_pls_base_launch_on_existing_daemons(map))) {
+            ORTE_ERROR_LOG(rc);
+            OBJ_RELEASE(map);
+            OBJ_DESTRUCT(&daemons);
+            return rc;
+        }
+    }
+    
     /*
      * Allocate a range of vpids for the daemons.
      */
-    num_nodes = opal_list_get_size(&nodes);
+    num_nodes = opal_list_get_size(&map->nodes);
     if (num_nodes == 0) {
-        return ORTE_ERR_BAD_PARAM;
+        /* nothing further to do - job must have been launched
+         * on existing daemons, so we can just return
+         */
+        OBJ_RELEASE(map);
+        OBJ_DESTRUCT(&daemons);
+        return ORTE_SUCCESS;
     }
     rc = orte_ns.reserve_range(0, num_nodes, &vpid);
     if (ORTE_SUCCESS != rc) {
         goto cleanup;
     }
+    start_vpid = vpid; 
 
+    /* setup the orted triggers for passing their launch info */
+    if (ORTE_SUCCESS != (rc = orte_smr.init_orted_stage_gates(jobid, num_nodes, NULL, NULL))) {
+        ORTE_ERROR_LOG(rc);
+        goto cleanup;
+    }
+    
     /* need integer value for command line parameter */
     asprintf(&jobid_string, "%lu", (unsigned long) jobid);
 
@@ -188,18 +233,18 @@ static int pls_slurm_launch(orte_jobid_t jobid)
     nodelist_argv = NULL;
     nodelist_argc = 0;
 
-    for (item =  opal_list_get_first(&nodes);
-         item != opal_list_get_end(&nodes);
+    for (item =  opal_list_get_first(&map->nodes);
+         item != opal_list_get_end(&map->nodes);
          item =  opal_list_get_next(item)) {
-        orte_ras_node_t* node = (orte_ras_node_t*)item;
+        orte_mapped_node_t* node = (orte_mapped_node_t*)item;
 
-        opal_argv_append(&nodelist_argc, &nodelist_argv, node->node_name);
+        opal_argv_append(&nodelist_argc, &nodelist_argv, node->nodename);
     }
     nodelist_flat = opal_argv_join(nodelist_argv, ',');
+    opal_argv_free(nodelist_argv);
     asprintf(&tmp, "--nodelist=%s", nodelist_flat);
     opal_argv_append(&argc, &argv, tmp);
     free(tmp);
-    free(nodelist_flat);
 
 
     /*
@@ -211,7 +256,7 @@ static int pls_slurm_launch(orte_jobid_t jobid)
     opal_argv_append(&argc, &argv, "--no-daemonize");
 
     /* check for debug flags */
-    orte_pls_base_proxy_mca_argv(&argc, &argv);
+    orte_pls_base_mca_argv(&argc, &argv);
 
     /* proxy information */
     opal_argv_append(&argc, &argv, "--bootproxy");
@@ -234,6 +279,8 @@ static int pls_slurm_launch(orte_jobid_t jobid)
         opal_output(0, "orte_pls_rsh: unable to create process name");
         goto cleanup;
     }
+    free(name);
+
     opal_argv_append(&argc, &argv, "--name");
     opal_argv_append(&argc, &argv, name_string);
     free(name_string);
@@ -290,114 +337,94 @@ static int pls_slurm_launch(orte_jobid_t jobid)
         }
     }
 
-    /* Bookkeeping -- save the node names */
+    /* Copy the prefix-directory specified in the
+       corresponding app_context.  If there are multiple,
+       different prefix's in the app context, complain (i.e., only
+       allow one --prefix option for the entire slurm run -- we
+       don't support different --prefix'es for different nodes in
+       the SLURM pls) */
     cur_prefix = NULL;
-    for (item =  opal_list_get_first(&nodes);
-         item != opal_list_get_end(&nodes);
-         item =  opal_list_get_next(item)) {
-        orte_ras_node_t* node = (orte_ras_node_t*)item;
-        orte_process_name_t* name;
-        opal_list_t map;
-        size_t num_processes;
+    for (i=0; i < map->num_apps; i++) {
+        char * app_prefix_dir = map->apps[i]->prefix_dir;
+         /* Check for already set cur_prefix_dir -- if different,
+           complain */
+        if (NULL != app_prefix_dir) {
+            if (NULL != cur_prefix &&
+                0 != strcmp (cur_prefix, app_prefix_dir)) {
+                opal_show_help("help-pls-slurm.txt", "multiple-prefixes",
+                               true, cur_prefix, app_prefix_dir);
+                return ORTE_ERR_FATAL;
+            }
 
-        OBJ_CONSTRUCT(&map, opal_list_t);
-        /* Get the mapping of this very node */
-        rc = orte_rmaps_base_get_node_map(orte_process_info.my_name->cellid,
-                                          jobid,
-                                          node->node_name,
-                                          &map);
-        if (ORTE_SUCCESS != rc) {
-            ORTE_ERROR_LOG(rc);
-            goto cleanup;
-        }
-
-        /* Copy the prefix-directory specified within the
-           corresponding app_context.  If there are multiple,
-           different prefix's in the app context, complain (i.e., only
-           allow one --prefix option for the entire slurm run -- we
-           don't support different --prefix'es for different nodes in
-           the SLURM pls) */
-        num_processes = 0;
-        for (item2 =  opal_list_get_first(&map);
-             item2 != opal_list_get_end(&map);
-             item2 =  opal_list_get_next(item2)) {
-            orte_rmaps_base_map_t* map = (orte_rmaps_base_map_t*) item2;
-            char * app_prefix_dir = map->app->prefix_dir;
-
-            /* Increment the number of processes allocated to this node
-             * This allows us to accurately test for oversubscription */
-            num_processes += map->num_procs;
-
-            /* Check for already set cur_prefix_dir -- if different,
-               complain */
-            if (NULL != app_prefix_dir) {
-                if (NULL != cur_prefix &&
-                    0 != strcmp (cur_prefix, app_prefix_dir)) {
-                    opal_show_help("help-pls-slurm.txt", "multiple-prefixes",
-                                   true, cur_prefix, app_prefix_dir);
-                    return ORTE_ERR_FATAL;
-                }
-
-                /* If not yet set, copy it; iff set, then it's the
-                   same anyway */
-                if (NULL == cur_prefix) {
-                    cur_prefix = strdup(map->app->prefix_dir);
-                    if (mca_pls_slurm_component.debug) {
-                        opal_output (0, "pls:slurm: Set prefix:%s",
-                                     cur_prefix);
-                    }
+            /* If not yet set, copy it; iff set, then it's the
+               same anyway */
+            if (NULL == cur_prefix) {
+                cur_prefix = strdup(app_prefix_dir);
+                if (mca_pls_slurm_component.debug) {
+                    opal_output (0, "pls:slurm: Set prefix:%s",
+                                 cur_prefix);
                 }
             }
         }
-
-        /* initialize daemons process name */
-        rc = orte_ns.create_process_name(&name, node->node_cellid, 0, vpid);
-        if (ORTE_SUCCESS != rc) {
-            ORTE_ERROR_LOG(rc);
-            goto cleanup;
-        }
-
-        /* save the daemons name on the node */
-        if (ORTE_SUCCESS !=
-            (rc = orte_pls_base_proxy_set_node_name(node, jobid, name))) {
-            ORTE_ERROR_LOG(rc);
-            goto cleanup;
-        }
-
-        vpid++;
-        free(name);
     }
 
+    /* setup the daemon info for each node */
+    vpid = start_vpid;
+    for (item = opal_list_get_first(&map->nodes);
+         item != opal_list_get_end(&map->nodes);
+         item = opal_list_get_next(item)) {
+        orte_mapped_node_t* node = (orte_mapped_node_t*)item;
+        
+        /* record the daemons info for this node */
+        dmn = OBJ_NEW(orte_pls_daemon_info_t);
+        dmn->active_job = jobid;
+        dmn->cell = node->cell;
+        dmn->nodename = strdup(node->nodename);
+        if (ORTE_SUCCESS != (rc = orte_ns.create_process_name(&(dmn->name), dmn->cell, 0, vpid))) {
+            ORTE_ERROR_LOG(rc);
+            goto cleanup;
+        }
+        opal_list_append(&daemons, &dmn->super);
+        vpid++;
+    }
+
+    /* store the daemon info on the registry */
+    if (ORTE_SUCCESS != (rc = orte_pls_base_store_active_daemons(&daemons))) {
+        ORTE_ERROR_LOG(rc);
+    }
+    
     /* setup environment */
     env = opal_argv_copy(environ);
     var = mca_base_param_environ_variable("seed", NULL, NULL);
     opal_setenv(var, "0", true, &env);
-
-#if 0
-    /* JMS What to do for sched_yield? */
-
-    /* set the progress engine schedule for this node.  if node_slots
-       is set to zero, then we default to NOT being oversubscribed */
-    if (node->node_slots > 0 &&
-        num_processes > node->node_slots) {
-        if (mca_pls_slurm_component.debug) {
-            opal_output(0, "pls:slurm: oversubscribed -- setting mpi_yield_when_idle to 1 (%d %d)",
-                        node->node_slots, num_processes);
-        }
-        var = mca_base_param_environ_variable("mpi", NULL, "yield_when_idle");
-        opal_setenv(var, "1", true, &env);
-    } else {
-        if (mca_pls_slurm_component.debug) {
-            opal_output(0, "pls:slurm: not oversubscribed -- setting mpi_yield_when_idle to 0");
-        }
-        var = mca_base_param_environ_variable("mpi", NULL, "yield_when_idle");
-        opal_setenv(var, "0", true, &env);
-    }
     free(var);
-#endif
+    var = mca_base_param_environ_variable("orte", "slurm", "nodelist");
+    opal_setenv(var, nodelist_flat, true, &env);
+    free(nodelist_flat);
+    free(var);
 
+    if (mca_pls_slurm_component.timing) {
+        if (0 != gettimeofday(&launchstart, NULL)) {
+            opal_output(0, "pls_slurm: could not obtain start time");
+        }        
+    }
+    
     /* exec the daemon */
     rc = pls_slurm_start_proc(argc, argv, env, cur_prefix);
+    
+    if (mca_pls_slurm_component.timing) {
+        if (0 != gettimeofday(&launchstop, NULL)) {
+             opal_output(0, "pls_slurm: could not obtain stop time");
+         } else {
+             opal_output(0, "pls_slurm: daemon block launch time is %ld usec",
+                         (launchstop.tv_sec - launchstart.tv_sec)*1000000 + 
+                         (launchstop.tv_usec - launchstart.tv_usec));
+             opal_output(0, "pls_slurm: total job launch time is %ld usec",
+                         (launchstop.tv_sec - joblaunchstart.tv_sec)*1000000 + 
+                         (launchstop.tv_usec - joblaunchstart.tv_usec));
+         }
+    }
+
     if (ORTE_SUCCESS != rc) {
         opal_output(0, "pls:slurm: start_procs returned error %d", rc);
         goto cleanup;
@@ -407,27 +434,78 @@ static int pls_slurm_launch(orte_jobid_t jobid)
     /* JMS: how do we catch when srun dies? */
 
 cleanup:
-    while (NULL != (item = opal_list_remove_first(&nodes))) {
-        OBJ_RELEASE(item);
-    }
-    OBJ_DESTRUCT(&nodes);
+    OBJ_RELEASE(map);
+    opal_argv_free(argv);
+    opal_argv_free(env);
 
-    while (NULL != (item = opal_list_remove_first(&mapping_list))) {
+    if(NULL != jobid_string) {
+        free(jobid_string);
+    }
+    
+    while (NULL != (item = opal_list_remove_first(&daemons))) {
         OBJ_RELEASE(item);
     }
-    OBJ_DESTRUCT(&mapping_list);
+    OBJ_DESTRUCT(&daemons);
+    
     return rc;
 }
 
 
-static int pls_slurm_terminate_job(orte_jobid_t jobid)
+static int pls_slurm_terminate_job(orte_jobid_t jobid, struct timeval *timeout, opal_list_t *attrs)
 {
-    if (0 != srun_pid) {
-        kill(srun_pid, SIGHUP);
-        /* JMS need appropriate code here to reap */
-        srun_pid = 0;
+    int rc;
+    opal_list_t daemons;
+    opal_list_item_t *item;
+    
+    /* construct the list of active daemons on this job */
+    OBJ_CONSTRUCT(&daemons, opal_list_t);
+    if (ORTE_SUCCESS != (rc = orte_pls_base_get_active_daemons(&daemons, jobid, attrs))) {
+        ORTE_ERROR_LOG(rc);
+        goto CLEANUP;
     }
-    return ORTE_SUCCESS;
+    
+    /* order them to kill their local procs for this job */
+    if (ORTE_SUCCESS != (rc = orte_pls_base_orted_kill_local_procs(&daemons, jobid, timeout))) {
+        ORTE_ERROR_LOG(rc);
+        goto CLEANUP;
+    }
+    
+CLEANUP:
+    while (NULL != (item = opal_list_remove_first(&daemons))) {
+        OBJ_RELEASE(item);
+    }
+    OBJ_DESTRUCT(&daemons);
+    return rc;
+}
+
+
+/**
+* Terminate the orteds for a given job
+ */
+static int pls_slurm_terminate_orteds(orte_jobid_t jobid, struct timeval *timeout, opal_list_t *attrs)
+{
+    int rc;
+    opal_list_t daemons;
+    opal_list_item_t *item;
+    
+    /* construct the list of active daemons on this job */
+    OBJ_CONSTRUCT(&daemons, opal_list_t);
+    if (ORTE_SUCCESS != (rc = orte_pls_base_get_active_daemons(&daemons, jobid, attrs))) {
+        ORTE_ERROR_LOG(rc);
+        goto CLEANUP;
+    }
+    
+    /* order them to go away */
+    if (ORTE_SUCCESS != (rc = orte_pls_base_orted_exit(&daemons, timeout))) {
+        ORTE_ERROR_LOG(rc);
+    }
+    
+CLEANUP:
+    while (NULL != (item = opal_list_remove_first(&daemons))) {
+        OBJ_RELEASE(item);
+    }
+    OBJ_DESTRUCT(&daemons);
+    return rc;
 }
 
 
@@ -437,8 +515,7 @@ static int pls_slurm_terminate_job(orte_jobid_t jobid)
  */
 static int pls_slurm_terminate_proc(const orte_process_name_t *name)
 {
-    opal_output(orte_pls_base.pls_output,
-                "pls:slurm:terminate_proc: not supported");
+    opal_output(0, "pls:slurm:terminate_proc: not supported");
     return ORTE_ERR_NOT_SUPPORTED;
 }
 
@@ -446,7 +523,7 @@ static int pls_slurm_terminate_proc(const orte_process_name_t *name)
 /**
  * Signal all the processes in the child srun by sending the signal directly to it
  */
-static int pls_slurm_signal_job(orte_jobid_t jobid, int32_t signal)
+static int pls_slurm_signal_job(orte_jobid_t jobid, int32_t signal, opal_list_t *attrs)
 {
     if (0 != srun_pid) {
         kill(srun_pid, (int)signal);
@@ -460,15 +537,35 @@ static int pls_slurm_signal_job(orte_jobid_t jobid, int32_t signal)
  */
 static int pls_slurm_signal_proc(const orte_process_name_t *name, int32_t signal)
 {
-    return orte_pls_base_proxy_signal_proc(name, signal);
+    opal_output(0, "pls:slurm:signal_proc: not supported");
+    return ORTE_ERR_NOT_SUPPORTED;
+}
+
+
+/**
+ * Cancel an operation involving comm to an orted
+ */
+int pls_slurm_cancel_operation(void)
+{
+    int rc;
+
+    if (ORTE_SUCCESS != (rc = orte_pls_base_orted_cancel_operation())) {
+        ORTE_ERROR_LOG(rc);
+    }
+    
+    return rc;
 }
 
 
 static int pls_slurm_finalize(void)
 {
-    /* cleanup any pending recvs */
-    orte_rml.recv_cancel(ORTE_RML_NAME_ANY, ORTE_RML_TAG_RMGR_CLNT);
+    int rc;
 
+    /* cleanup any pending recvs */
+    if (ORTE_SUCCESS != (rc = orte_pls_base_comm_stop())) {
+        ORTE_ERROR_LOG(rc);
+    }
+    
     return ORTE_SUCCESS;
 }
 
@@ -476,7 +573,7 @@ static int pls_slurm_finalize(void)
 static int pls_slurm_start_proc(int argc, char **argv, char **env,
                                 char *prefix)
 {
-    int fd;
+    int fd, id, debug_daemons;
     char *exec_argv = opal_path_findv(argv[0], 0, env, NULL);
 
     if (NULL == exec_argv) {
@@ -485,8 +582,7 @@ static int pls_slurm_start_proc(int argc, char **argv, char **env,
 
     srun_pid = fork();
     if (-1 == srun_pid) {
-        opal_output(orte_pls_base.pls_output,
-                    "pls:slurm:start_proc: fork failed");
+        opal_output(0, "pls:slurm:start_proc: fork failed");
         return ORTE_ERR_IN_ERRNO;
     } else if (0 == srun_pid) {
         char *bin_base = NULL, *lib_base = NULL;
@@ -496,12 +592,11 @@ static int pls_slurm_start_proc(int argc, char **argv, char **env,
            explaining all the rationale for how / why we're doing
            this. */
 
-        lib_base = opal_basename(OPAL_LIBDIR);
-        bin_base = opal_basename(OPAL_BINDIR);
+        lib_base = opal_basename(opal_install_dirs.libdir);
+        bin_base = opal_basename(opal_install_dirs.bindir);
 
         /* If we have a prefix, then modify the PATH and
-           LD_LIBRARY_PATH environment variables.  We're already in
-           the child process, so it's ok to modify environ. */
+           LD_LIBRARY_PATH environment variables.  */
         if (NULL != prefix) {
             char *oldenv, *newenv;
 
@@ -512,7 +607,7 @@ static int pls_slurm_start_proc(int argc, char **argv, char **env,
             } else {
                 asprintf(&newenv, "%s/%s", prefix, bin_base);
             }
-            opal_setenv("PATH", newenv, true, &environ);
+            opal_setenv("PATH", newenv, true, &env);
             if (mca_pls_slurm_component.debug) {
                 opal_output(0, "pls:slurm: reset PATH: %s", newenv);
             }
@@ -525,7 +620,7 @@ static int pls_slurm_start_proc(int argc, char **argv, char **env,
             } else {
                 asprintf(&newenv, "%s/%s", prefix, lib_base);
             }
-            opal_setenv("LD_LIBRARY_PATH", newenv, true, &environ);
+            opal_setenv("LD_LIBRARY_PATH", newenv, true, &env);
             if (mca_pls_slurm_component.debug) {
                 opal_output(0, "pls:slurm: reset LD_LIBRARY_PATH: %s",
                             newenv);
@@ -533,10 +628,19 @@ static int pls_slurm_start_proc(int argc, char **argv, char **env,
             free(newenv);
         }
 
-        /* When not in debug mode, tie stdout/stderr to dev null so we
-           don't see messages from orted */
-        if (!mca_pls_slurm_component.debug) {
-            fd = open("/dev/null", O_CREAT|O_WRONLY|O_TRUNC, 0666);
+        fd = open("/dev/null", O_CREAT|O_WRONLY|O_TRUNC, 0666);
+        if(fd > 0) {
+            dup2(fd, 0);
+        }
+
+        /* When not in debug mode and --debug-daemons was not passed,
+         * tie stdout/stderr to dev null so we don't see messages from orted */
+        id = mca_base_param_find("orte", "debug", "daemons");
+        if(id < 0) {
+            id = mca_base_param_register_int("orte", "debug", "daemons", NULL, 0);
+        }
+        mca_base_param_lookup_int(id, &debug_daemons);
+        if (0 == mca_pls_slurm_component.debug && 0 == debug_daemons) {
             if (fd >= 0) {
                 if (fd != 1) {
                     dup2(fd,1);
@@ -544,25 +648,28 @@ static int pls_slurm_start_proc(int argc, char **argv, char **env,
                 if (fd != 2) {
                     dup2(fd,2);
                 }
-                if (fd > 2) {
-                    close(fd);
-                }
             }
         }
 
+        if (fd > 2) {
+            close(fd);
+        }
+
         /* get the srun process out of orterun's process group so that
            signals sent from the shell (like those resulting from
            cntl-c) don't get sent to srun */
         setpgid(0, 0);
 
-        opal_output(orte_pls_base.pls_output,
-                    "pls:slurm:start_proc: exec failed");
         execve(exec_argv, argv, env);
+
+        opal_output(0, "pls:slurm:start_proc: exec failed");
         /* don't return - need to exit - returning would be bad -
            we're not in the calling process anymore */
         exit(1);
     }
 
+    free(exec_argv);
+
     /* just in case, make sure that the srun process is not in our
        process group any more.  Stevens says always do this on both
        sides of the fork... */
