@@ -25,14 +25,15 @@
 #include "opal/util/argv.h"
 
 #include "orte/mca/errmgr/errmgr.h"
-#include "orte/mca/soh/soh_types.h"
+#include "orte/mca/smr/smr_types.h"
 #include "orte/mca/gpr/gpr.h"
 #include "orte/mca/ns/ns.h"
-#include "orte/mca/ras/base/ras_base_node.h"
+#include "orte/mca/ras/base/ras_private.h"
 
 static void orte_ras_base_node_construct(orte_ras_node_t* node)
 {
     node->node_name = NULL;
+    node->launch_id = -1;
     node->node_arch = NULL;
     node->node_cellid = 0;
     node->node_state = ORTE_NODE_STATE_UNKNOWN;
@@ -71,9 +72,23 @@ OBJ_CLASS_INSTANCE(
 
 int orte_ras_base_node_query(opal_list_t* nodes)
 {
-    size_t i, cnt, *sptr;
+    char* keys[] = {
+        ORTE_NODE_NAME_KEY,
+        ORTE_NODE_LAUNCH_ID_KEY,
+        ORTE_NODE_ARCH_KEY,
+        ORTE_NODE_STATE_KEY,
+        ORTE_NODE_SLOTS_KEY,
+        ORTE_NODE_SLOTS_IN_USE_KEY,
+        ORTE_NODE_SLOTS_ALLOC_KEY,
+        ORTE_NODE_SLOTS_MAX_KEY,
+        ORTE_NODE_USERNAME_KEY,
+        ORTE_CELLID_KEY,
+        NULL
+    };
+    orte_std_cntr_t i, cnt, *sptr;
     orte_node_state_t *nsptr;
     orte_cellid_t *cptr;
+    int32_t *i32;
     orte_gpr_value_t** values;
     int rc;
 
@@ -82,7 +97,7 @@ int orte_ras_base_node_query(opal_list_t* nodes)
         ORTE_GPR_KEYS_OR|ORTE_GPR_TOKENS_OR,
         ORTE_NODE_SEGMENT,
         NULL,
-        NULL,
+        keys,
         &cnt,
         &values);
     if(ORTE_SUCCESS != rc) {
@@ -94,7 +109,7 @@ int orte_ras_base_node_query(opal_list_t* nodes)
     for(i=0; i<cnt; i++) {
         orte_gpr_value_t* value = values[i];
         orte_ras_node_t* node = OBJ_NEW(orte_ras_node_t);
-        size_t k;
+        orte_std_cntr_t k;
 
         for(k=0; k<value->cnt; k++) {
             orte_gpr_keyval_t* keyval = value->keyvals[k];
@@ -108,6 +123,14 @@ int orte_ras_base_node_query(opal_list_t* nodes)
                 }
                 continue;
             }
+            if(strcmp(keyval->key, ORTE_NODE_LAUNCH_ID_KEY) == 0) {
+                if (ORTE_SUCCESS != (rc = orte_dss.get((void**)&i32, keyval->value, ORTE_INT32))) {
+                    ORTE_ERROR_LOG(rc);
+                    continue;
+                }
+                node->launch_id = *i32;
+                continue;
+            }
             if(strcmp(keyval->key, ORTE_NODE_ARCH_KEY) == 0) {
                 /* we use the dss.copy function here instead of strdup because that function
                  * automatically protects us against a NULL (or zero-length) string
@@ -127,7 +150,7 @@ int orte_ras_base_node_query(opal_list_t* nodes)
                 continue;
             }
             if(strcmp(keyval->key, ORTE_NODE_SLOTS_KEY) == 0) {
-                if (ORTE_SUCCESS != (rc = orte_dss.get((void**)&sptr, keyval->value, ORTE_SIZE))) {
+                if (ORTE_SUCCESS != (rc = orte_dss.get((void**)&sptr, keyval->value, ORTE_STD_CNTR))) {
                     ORTE_ERROR_LOG(rc);
                     continue;
                 }
@@ -135,7 +158,7 @@ int orte_ras_base_node_query(opal_list_t* nodes)
                 continue;
             }
             if(strcmp(keyval->key, ORTE_NODE_SLOTS_IN_USE_KEY) == 0) {
-                if (ORTE_SUCCESS != (rc = orte_dss.get((void**)&sptr, keyval->value, ORTE_SIZE))) {
+                if (ORTE_SUCCESS != (rc = orte_dss.get((void**)&sptr, keyval->value, ORTE_STD_CNTR))) {
                     ORTE_ERROR_LOG(rc);
                     continue;
                 }
@@ -143,7 +166,7 @@ int orte_ras_base_node_query(opal_list_t* nodes)
                 continue;
             }
             if(strncmp(keyval->key, ORTE_NODE_SLOTS_ALLOC_KEY, strlen(ORTE_NODE_SLOTS_ALLOC_KEY)) == 0) {
-                if (ORTE_SUCCESS != (rc = orte_dss.get((void**)&sptr, keyval->value, ORTE_SIZE))) {
+                if (ORTE_SUCCESS != (rc = orte_dss.get((void**)&sptr, keyval->value, ORTE_STD_CNTR))) {
                     ORTE_ERROR_LOG(rc);
                     continue;
                 }
@@ -151,7 +174,7 @@ int orte_ras_base_node_query(opal_list_t* nodes)
                 continue;
             }
             if(strcmp(keyval->key, ORTE_NODE_SLOTS_MAX_KEY) == 0) {
-                if (ORTE_SUCCESS != (rc = orte_dss.get((void**)&sptr, keyval->value, ORTE_SIZE))) {
+                if (ORTE_SUCCESS != (rc = orte_dss.get((void**)&sptr, keyval->value, ORTE_STD_CNTR))) {
                     ORTE_ERROR_LOG(rc);
                     continue;
                 }
@@ -193,6 +216,7 @@ int orte_ras_base_node_query_alloc(opal_list_t* nodes, orte_jobid_t jobid)
 {
     char* keys[] = {
         ORTE_NODE_NAME_KEY,
+        ORTE_NODE_LAUNCH_ID_KEY,
         ORTE_NODE_ARCH_KEY,
         ORTE_NODE_STATE_KEY,
         ORTE_NODE_SLOTS_KEY,
@@ -203,12 +227,13 @@ int orte_ras_base_node_query_alloc(opal_list_t* nodes, orte_jobid_t jobid)
         ORTE_CELLID_KEY,
         NULL
     };
-    size_t i, cnt, keys_len;
+    orte_std_cntr_t i, cnt, keys_len;
     orte_gpr_value_t** values;
     char* jobid_str;
-    size_t *sptr;
+    orte_std_cntr_t *sptr;
     orte_node_state_t *nsptr;
     orte_cellid_t *cptr;
+    int32_t *i32;
     int rc, alloc_key_posn=5;
 
     if(ORTE_SUCCESS != (rc = orte_ns.convert_jobid_to_string(&jobid_str, jobid))) {
@@ -217,7 +242,7 @@ int orte_ras_base_node_query_alloc(opal_list_t* nodes, orte_jobid_t jobid)
     }
 
     asprintf(&keys[alloc_key_posn], "%s-%s", ORTE_NODE_SLOTS_ALLOC_KEY, jobid_str);
-    keys_len = strlen(keys[alloc_key_posn]);
+    keys_len = (orte_std_cntr_t)strlen(keys[alloc_key_posn]);
     free(jobid_str);
 
     /* query selected node entries */
@@ -237,7 +262,7 @@ int orte_ras_base_node_query_alloc(opal_list_t* nodes, orte_jobid_t jobid)
     for(i=0; i<cnt; i++) {
         orte_gpr_value_t* value = values[i];
         orte_ras_node_t* node;
-        size_t k;
+        orte_std_cntr_t k;
         bool found;
 
         found = false;
@@ -266,6 +291,14 @@ int orte_ras_base_node_query_alloc(opal_list_t* nodes, orte_jobid_t jobid)
                 }
                 continue;
             }
+            if(strcmp(keyval->key, ORTE_NODE_LAUNCH_ID_KEY) == 0) {
+                if (ORTE_SUCCESS != (rc = orte_dss.get((void**)&i32, keyval->value, ORTE_INT32))) {
+                    ORTE_ERROR_LOG(rc);
+                    continue;
+                }
+                node->launch_id = *i32;
+                continue;
+            }
             if(strcmp(keyval->key, ORTE_NODE_ARCH_KEY) == 0) {
                 /* we use the dss.copy function here instead of strdup because that function
                  * automatically protects us against a NULL (or zero-length) string
@@ -285,7 +318,7 @@ int orte_ras_base_node_query_alloc(opal_list_t* nodes, orte_jobid_t jobid)
                 continue;
             }
             if(strcmp(keyval->key, ORTE_NODE_SLOTS_KEY) == 0) {
-                if (ORTE_SUCCESS != (rc = orte_dss.get((void**)&sptr, keyval->value, ORTE_SIZE))) {
+                if (ORTE_SUCCESS != (rc = orte_dss.get((void**)&sptr, keyval->value, ORTE_STD_CNTR))) {
                     ORTE_ERROR_LOG(rc);
                     continue;
                 }
@@ -293,7 +326,7 @@ int orte_ras_base_node_query_alloc(opal_list_t* nodes, orte_jobid_t jobid)
                 continue;
             }
             if(strcmp(keyval->key, ORTE_NODE_SLOTS_IN_USE_KEY) == 0) {
-                if (ORTE_SUCCESS != (rc = orte_dss.get((void**)&sptr, keyval->value, ORTE_SIZE))) {
+                if (ORTE_SUCCESS != (rc = orte_dss.get((void**)&sptr, keyval->value, ORTE_STD_CNTR))) {
                     ORTE_ERROR_LOG(rc);
                     continue;
                 }
@@ -301,7 +334,7 @@ int orte_ras_base_node_query_alloc(opal_list_t* nodes, orte_jobid_t jobid)
                 continue;
             }
             if(strncmp(keyval->key, keys[alloc_key_posn], keys_len) == 0) {
-                if (ORTE_SUCCESS != (rc = orte_dss.get((void**)&sptr, keyval->value, ORTE_SIZE))) {
+                if (ORTE_SUCCESS != (rc = orte_dss.get((void**)&sptr, keyval->value, ORTE_STD_CNTR))) {
                     ORTE_ERROR_LOG(rc);
                     continue;
                 }
@@ -309,7 +342,7 @@ int orte_ras_base_node_query_alloc(opal_list_t* nodes, orte_jobid_t jobid)
                 continue;
             }
             if(strcmp(keyval->key, ORTE_NODE_SLOTS_MAX_KEY) == 0) {
-                if (ORTE_SUCCESS != (rc = orte_dss.get((void**)&sptr, keyval->value, ORTE_SIZE))) {
+                if (ORTE_SUCCESS != (rc = orte_dss.get((void**)&sptr, keyval->value, ORTE_STD_CNTR))) {
                     ORTE_ERROR_LOG(rc);
                     continue;
                 }
@@ -335,7 +368,19 @@ int orte_ras_base_node_query_alloc(opal_list_t* nodes, orte_jobid_t jobid)
                 continue;
             }
         }
-        opal_list_append(nodes, &node->super);
+        /* check to see if any slots were reserved on this node for us
+         * The "get" command will return data from ALL nodes on the node
+         * segment. We ONLY want to include here nodes that are assigned
+         * to the specified job - i.e., nodes that have a node_slots_alloc_key
+         * for this jobid. If that is the case, then the node_slots_alloc will be
+         * set to a value greater than 0
+         */
+        if (0 < node->node_slots_alloc) {
+            opal_list_append(nodes, &node->super);
+        } else {
+            /* no slots were allocated to us on this node */
+            OBJ_RELEASE(node);
+        }
         OBJ_RELEASE(value);
     }
 
@@ -351,11 +396,25 @@ int orte_ras_base_node_query_alloc(opal_list_t* nodes, orte_jobid_t jobid)
 
 orte_ras_node_t* orte_ras_base_node_lookup(orte_cellid_t cellid, const char* node_name)
 {
+    char* keys[] = {
+        ORTE_NODE_NAME_KEY,
+        ORTE_NODE_LAUNCH_ID_KEY,
+        ORTE_NODE_ARCH_KEY,
+        ORTE_NODE_STATE_KEY,
+        ORTE_NODE_SLOTS_KEY,
+        ORTE_NODE_SLOTS_IN_USE_KEY,
+        ORTE_NODE_SLOTS_ALLOC_KEY,
+        ORTE_NODE_SLOTS_MAX_KEY,
+        ORTE_NODE_USERNAME_KEY,
+        ORTE_CELLID_KEY,
+        NULL
+    };
     orte_ras_node_t* node = NULL;
-    size_t i, cnt, num_tokens;
-    size_t *sptr;
+    orte_std_cntr_t i, cnt, num_tokens;
+    orte_std_cntr_t *sptr;
     orte_cellid_t *cptr;
     orte_node_state_t *nsptr;
+    int32_t *i32;
     orte_gpr_value_t** values;
     char** tokens = NULL;
     int rc;
@@ -371,7 +430,7 @@ orte_ras_node_t* orte_ras_base_node_lookup(orte_cellid_t cellid, const char* nod
         ORTE_GPR_KEYS_OR|ORTE_GPR_TOKENS_OR,
         ORTE_NODE_SEGMENT,
         tokens,
-        NULL,
+        keys,
         &cnt,
         &values);
     if(ORTE_SUCCESS != rc) {
@@ -382,7 +441,7 @@ orte_ras_node_t* orte_ras_base_node_lookup(orte_cellid_t cellid, const char* nod
     /* parse the response */
     for(i=0; i<cnt; i++) {
         orte_gpr_value_t* value = values[i];
-        size_t k;
+        orte_std_cntr_t k;
         node = OBJ_NEW(orte_ras_node_t);
 
         for(k=0; k<value->cnt; k++) {
@@ -397,6 +456,14 @@ orte_ras_node_t* orte_ras_base_node_lookup(orte_cellid_t cellid, const char* nod
                 }
                 continue;
             }
+            if(strcmp(keyval->key, ORTE_NODE_LAUNCH_ID_KEY) == 0) {
+                if (ORTE_SUCCESS != (rc = orte_dss.get((void**)&i32, keyval->value, ORTE_INT32))) {
+                    ORTE_ERROR_LOG(rc);
+                    continue;
+                }
+                node->launch_id = *i32;
+                continue;
+            }
             if(strcmp(keyval->key, ORTE_NODE_ARCH_KEY) == 0) {
                 /* we use the dss.copy function here instead of strdup because that function
                  * automatically protects us against a NULL (or zero-length) string
@@ -416,7 +483,7 @@ orte_ras_node_t* orte_ras_base_node_lookup(orte_cellid_t cellid, const char* nod
                 continue;
             }
             if(strcmp(keyval->key, ORTE_NODE_SLOTS_KEY) == 0) {
-                if (ORTE_SUCCESS != (rc = orte_dss.get((void**)&sptr, keyval->value, ORTE_SIZE))) {
+                if (ORTE_SUCCESS != (rc = orte_dss.get((void**)&sptr, keyval->value, ORTE_STD_CNTR))) {
                     ORTE_ERROR_LOG(rc);
                     continue;
                 }
@@ -424,7 +491,7 @@ orte_ras_node_t* orte_ras_base_node_lookup(orte_cellid_t cellid, const char* nod
                 continue;
             }
             if(strcmp(keyval->key, ORTE_NODE_SLOTS_IN_USE_KEY) == 0) {
-                if (ORTE_SUCCESS != (rc = orte_dss.get((void**)&sptr, keyval->value, ORTE_SIZE))) {
+                if (ORTE_SUCCESS != (rc = orte_dss.get((void**)&sptr, keyval->value, ORTE_STD_CNTR))) {
                     ORTE_ERROR_LOG(rc);
                     continue;
                 }
@@ -432,7 +499,7 @@ orte_ras_node_t* orte_ras_base_node_lookup(orte_cellid_t cellid, const char* nod
                 continue;
             }
             if(strncmp(keyval->key, ORTE_NODE_SLOTS_ALLOC_KEY, strlen(ORTE_NODE_SLOTS_ALLOC_KEY)) == 0) {
-                if (ORTE_SUCCESS != (rc = orte_dss.get((void**)&sptr, keyval->value, ORTE_SIZE))) {
+                if (ORTE_SUCCESS != (rc = orte_dss.get((void**)&sptr, keyval->value, ORTE_STD_CNTR))) {
                     ORTE_ERROR_LOG(rc);
                     continue;
                 }
@@ -440,7 +507,7 @@ orte_ras_node_t* orte_ras_base_node_lookup(orte_cellid_t cellid, const char* nod
                 continue;
             }
             if(strcmp(keyval->key, ORTE_NODE_SLOTS_MAX_KEY) == 0) {
-                if (ORTE_SUCCESS != (rc = orte_dss.get((void**)&sptr, keyval->value, ORTE_SIZE))) {
+                if (ORTE_SUCCESS != (rc = orte_dss.get((void**)&sptr, keyval->value, ORTE_STD_CNTR))) {
                     ORTE_ERROR_LOG(rc);
                     continue;
                 }
@@ -485,9 +552,10 @@ int orte_ras_base_node_insert(opal_list_t* nodes)
     opal_list_item_t* item;
     orte_gpr_value_t **values;
     int rc;
-    size_t num_values, i, j;
+    orte_std_cntr_t num_values, i, j;
     char *keys[] = {
         ORTE_NODE_NAME_KEY,
+        ORTE_NODE_LAUNCH_ID_KEY,
         ORTE_NODE_ARCH_KEY,
         ORTE_NODE_STATE_KEY,
         ORTE_CELLID_KEY,
@@ -498,17 +566,18 @@ int orte_ras_base_node_insert(opal_list_t* nodes)
         };
     orte_data_type_t types[] = {
         ORTE_STRING,
+        ORTE_INT32,
         ORTE_STRING,
         ORTE_NODE_STATE,
         ORTE_CELLID,
-        ORTE_SIZE,
-        ORTE_SIZE,
-        ORTE_SIZE,
+        ORTE_STD_CNTR,
+        ORTE_STD_CNTR,
+        ORTE_STD_CNTR,
         ORTE_STRING
     };
     orte_ras_node_t* node;
 
-    num_values = opal_list_get_size(nodes);
+    num_values = (orte_std_cntr_t)opal_list_get_size(nodes);
     if (0 >= num_values) {
         ORTE_ERROR_LOG(ORTE_ERR_BAD_PARAM);
         return ORTE_ERR_BAD_PARAM;
@@ -523,7 +592,7 @@ int orte_ras_base_node_insert(opal_list_t* nodes)
     for (i=0; i < num_values; i++) {
         if (ORTE_SUCCESS != (rc = orte_gpr.create_value(&(values[i]),
                                     ORTE_GPR_OVERWRITE | ORTE_GPR_TOKENS_AND,
-                                    ORTE_NODE_SEGMENT, 8, 0))) {
+                                    ORTE_NODE_SEGMENT, 9, 0))) {
             ORTE_ERROR_LOG(rc);
             for (j=0; j < i; j++) {
                 OBJ_RELEASE(values[j]);
@@ -544,6 +613,12 @@ int orte_ras_base_node_insert(opal_list_t* nodes)
             goto cleanup;
         }
         
+        ++j;
+        if (ORTE_SUCCESS != (rc = orte_gpr.create_keyval(&(values[i]->keyvals[j]), keys[j], types[j], &(node->launch_id)))) {
+            ORTE_ERROR_LOG(rc);
+            goto cleanup;
+        }
+        
         ++j;
         if (ORTE_SUCCESS != (rc = orte_gpr.create_keyval(&(values[i]->keyvals[j]), keys[j], types[j], node->node_arch))) {
             ORTE_ERROR_LOG(rc);
@@ -614,11 +689,11 @@ int orte_ras_base_node_delete(opal_list_t* nodes)
 {
     opal_list_item_t* item;
     int rc;
-    size_t i, num_values, num_tokens;
+    orte_std_cntr_t i, num_values, num_tokens;
     orte_ras_node_t* node;
     char** tokens;
 
-    num_values = opal_list_get_size(nodes);
+    num_values = (orte_std_cntr_t)opal_list_get_size(nodes);
     if (0 >= num_values) {
         ORTE_ERROR_LOG(ORTE_ERR_BAD_PARAM);
         return ORTE_ERR_BAD_PARAM;
@@ -663,17 +738,20 @@ int orte_ras_base_node_assign(opal_list_t* nodes, orte_jobid_t jobid)
     opal_list_item_t* item;
     orte_gpr_value_t **values;
     int rc;
-    size_t num_values, i, j;
+    orte_std_cntr_t num_values, i, j, total_slots;
     orte_ras_node_t* node;
-    char* jobid_str, *key;
+    char* jobid_str, *key=NULL, *segment;
 
-    num_values = opal_list_get_size(nodes);
+    num_values = (orte_std_cntr_t)opal_list_get_size(nodes);
     if (0 >= num_values) {
         ORTE_ERROR_LOG(ORTE_ERR_BAD_PARAM);
         return ORTE_ERR_BAD_PARAM;
     }
 
-    values = (orte_gpr_value_t**)malloc(num_values * sizeof(orte_gpr_value_t*));
+    /* get one value more than needed for these nodes so we can store the total number of
+     * slots being assigned to this jobid in the job segment
+     */
+    values = (orte_gpr_value_t**)malloc((1+num_values) * sizeof(orte_gpr_value_t*));
     if (NULL == values) {
         ORTE_ERROR_LOG(ORTE_ERR_OUT_OF_RESOURCE);
         return ORTE_ERR_OUT_OF_RESOURCE;
@@ -691,6 +769,17 @@ int orte_ras_base_node_assign(opal_list_t* nodes, orte_jobid_t jobid)
         }
     }
 
+    /* setup the allocation key */
+    if (ORTE_SUCCESS != (rc = orte_ns.convert_jobid_to_string(&jobid_str, jobid))) {
+        ORTE_ERROR_LOG(rc);
+        goto cleanup;
+    }
+    asprintf(&key, "%s-%s", ORTE_NODE_SLOTS_ALLOC_KEY, jobid_str);
+    free(jobid_str);
+    
+    /* initialize the total slots */
+    total_slots = 0;
+    
     for(i=0, item =  opal_list_get_first(nodes);
         i < num_values && item != opal_list_get_end(nodes);
         i++, item = opal_list_get_next(item)) {
@@ -699,11 +788,6 @@ int orte_ras_base_node_assign(opal_list_t* nodes, orte_jobid_t jobid)
         if(node->node_slots_alloc == 0)
             continue;
 
-        if (ORTE_SUCCESS != (rc = orte_ns.convert_jobid_to_string(&jobid_str, jobid))) {
-            ORTE_ERROR_LOG(rc);
-            goto cleanup;
-        }
-
         /* setup index/keys for this node */
         rc = orte_schema.get_node_tokens(&(values[i]->tokens), &(values[i]->num_tokens), node->node_cellid, node->node_name);
         if (ORTE_SUCCESS != rc) {
@@ -713,27 +797,53 @@ int orte_ras_base_node_assign(opal_list_t* nodes, orte_jobid_t jobid)
         }
 
         /* setup node key/value pairs */
-        asprintf(&key, "%s-%s", ORTE_NODE_SLOTS_ALLOC_KEY, jobid_str);
-        free(jobid_str);
-        if (ORTE_SUCCESS != (rc = orte_gpr.create_keyval(&(values[i]->keyvals[0]), key, ORTE_SIZE, &(node->node_slots_alloc)))) {
+        if (ORTE_SUCCESS != (rc = orte_gpr.create_keyval(&(values[i]->keyvals[0]), key, ORTE_STD_CNTR, &(node->node_slots_alloc)))) {
             ORTE_ERROR_LOG(rc);
             free(key);
             goto cleanup;
         }
-        free(key);
+        
+        /* add the slots to our total */
+        total_slots += node->node_slots;
     }
-
-    /* try the insert */
-    if (ORTE_SUCCESS != (rc = orte_gpr.put(num_values, values))) {
+    
+    /* setup to store the total number of slots */
+    if (ORTE_SUCCESS != (rc = orte_schema.get_job_segment_name(&segment, jobid))) {
+        ORTE_ERROR_LOG(rc);
+        goto cleanup;
+    }
+    
+    if (ORTE_SUCCESS != (rc = orte_gpr.create_value(&(values[num_values]), ORTE_GPR_OVERWRITE | ORTE_GPR_TOKENS_AND,
+                                                    segment, 1, 1))) {
+        ORTE_ERROR_LOG(rc);
+        free(segment);
+        goto cleanup;
+    }
+    free(segment);
+    
+    /* enter the value */
+    if (ORTE_SUCCESS != (rc = orte_gpr.create_keyval(&(values[num_values]->keyvals[0]),
+                                                     ORTE_JOB_TOTAL_SLOTS_ALLOC_KEY, ORTE_STD_CNTR, &total_slots))) {
+        ORTE_ERROR_LOG(rc);
+        goto cleanup;
+    }
+    
+    /* put the value in the JOB_GLOBALS container */
+    values[num_values]->tokens[0] = strdup(ORTE_JOB_GLOBALS);
+    
+    /* do the insert */
+    if (ORTE_SUCCESS != (rc = orte_gpr.put((1+num_values), values))) {
         ORTE_ERROR_LOG(rc);
     }
 
 cleanup:
-    for (j=0; j < num_values; j++) {
+    for (j=0; j < (1+num_values); j++) {
         OBJ_RELEASE(values[j]);
     }
     if (NULL != values) free(values);
 
+    if (NULL != key) free(key);
+
     return rc;
 }
 
@@ -753,7 +863,7 @@ int orte_ras_base_node_segment_empty(bool *empty)
         return ret;
     }
 
-    *empty = opal_list_is_empty(&nodes) ? true : false;
+    *empty = opal_list_is_empty(&nodes);
 
     /* Free the list */
 
